@@ -112,6 +112,48 @@ static fsal_status_t ceph_fsal_lookup(struct fsal_obj_handle *dir_pub,
 	return fsalstat(0, 0);
 }
 
+static int
+ceph_fsal_get_sec_label(struct ceph_handle *handle, struct attrlist *attrs)
+{
+	int rc = 0;
+	struct ceph_export *export =
+		container_of(op_ctx->fsal_export, struct ceph_export, export);
+
+	if (FSAL_TEST_MASK(attrs->request_mask, ATTR4_SEC_LABEL) &&
+	    op_ctx_export_has_option(EXPORT_OPTION_SECLABEL_SET)) {
+		char label[NFS4_OPAQUE_LIMIT];
+		struct user_cred root_creds = {};
+
+		/*
+		 * It's possible that the user won't have permission to fetch
+		 * the xattrs, so use root creds to get them since it's
+		 * supposed to be part of the inode metadata.
+		 */
+		rc = fsal_ceph_ll_getxattr(export->cmount, handle->i,
+					   export->sec_label_xattr, label,
+					   NFS4_OPAQUE_LIMIT, &root_creds);
+		if (rc < 0) {
+			/* If there's no label then just do zero-length one */
+			if (rc != -ENODATA)
+				goto out_err;
+			rc = 0;
+		}
+
+		attrs->sec_label.slai_data.slai_data_len = rc;
+		gsh_free(attrs->sec_label.slai_data.slai_data_val);
+		if (rc > 0) {
+			attrs->sec_label.slai_data.slai_data_val =
+				gsh_memdup(label, rc);
+			FSAL_SET_MASK(attrs->valid_mask, ATTR4_SEC_LABEL);
+		} else {
+			attrs->sec_label.slai_data.slai_data_val = NULL;
+			FSAL_UNSET_MASK(attrs->valid_mask, ATTR4_SEC_LABEL);
+		}
+	}
+out_err:
+	return rc;
+}
+
 /**
  * @brief Read a directory
  *
@@ -180,6 +222,13 @@ static fsal_status_t ceph_fsal_readdir(struct fsal_obj_handle *dir_pub,
 			/* skip . and .. */
 			if ((strcmp(de.d_name, ".") == 0)
 			    || (strcmp(de.d_name, "..") == 0)) {
+				/* Deref inode here as we reference inode in
+				 * libcephfs readdir_r_cb. The other inodes
+				 * gets deref in deconstruct_handle.
+				 */
+				if (i != NULL)
+					ceph_ll_put(export->cmount, i);
+
 				continue;
 			}
 
@@ -188,6 +237,14 @@ static fsal_status_t ceph_fsal_readdir(struct fsal_obj_handle *dir_pub,
 			fsal_prepare_attrs(&attrs, attrmask);
 			ceph2fsal_attributes(&stx, &attrs);
 
+			rc = ceph_fsal_get_sec_label(obj, &attrs);
+			if (rc < 0) {
+				fsal_status = ceph2fsal_error(rc);
+				if (i != NULL)
+					ceph_ll_put(export->cmount, i);
+				goto closedir;
+			}
+
 			cb_rc = cb(de.d_name, &obj->handle, &attrs, dir_state,
 					de.d_off);
 
@@ -288,6 +345,14 @@ static fsal_status_t ceph_fsal_mkdir(struct fsal_obj_handle *dir_hdl,
 				     fsal_err_txt(status));
 			(*new_obj)->obj_ops->release(*new_obj);
 			*new_obj = NULL;
+		} else if (attrs_out != NULL) {
+			/*
+			 * We ignore errors here. The mkdir and setattr
+			 * succeeded, so we don't want to return error if the
+			 * getattrs fails. We'll just return no attributes
+			 * in that case.
+			 */
+			(*new_obj)->obj_ops->getattrs(*new_obj, attrs_out);
 		}
 	} else {
 		status = fsalstat(ERR_FSAL_NO_ERROR, 0);
@@ -577,18 +642,21 @@ static fsal_status_t ceph_fsal_getattrs(struct fsal_obj_handle *handle_pub,
 
 	rc = fsal_ceph_ll_getattr(export->cmount, handle->i, &stx,
 				CEPH_STATX_ATTR_MASK, op_ctx->creds);
-	LogDebug(COMPONENT_FSAL, "getattr returned %d", rc);
-	if (rc < 0) {
-		if (attrs->request_mask & ATTR_RDATTR_ERR) {
-			/* Caller asked for error to be visible. */
-			attrs->valid_mask = ATTR_RDATTR_ERR;
-		}
-		return ceph2fsal_error(rc);
-	}
+	if (rc < 0)
+		goto out_err;
+
+	rc = ceph_fsal_get_sec_label(handle, attrs);
+	if (rc < 0)
+		goto out_err;
 
 	ceph2fsal_attributes(&stx, attrs);
-
 	return fsalstat(ERR_FSAL_NO_ERROR, 0);
+out_err:
+	if (attrs->request_mask & ATTR_RDATTR_ERR) {
+		/* Caller asked for error to be visible. */
+		attrs->valid_mask = ATTR_RDATTR_ERR;
+	}
+	return ceph2fsal_error(rc);
 }
 
 /**
@@ -771,7 +839,7 @@ static fsal_status_t ceph_open_my_fd(struct ceph_handle *myself,
 		     "fd = %p, new openflags = %x",
 		     my_fd->fd, openflags);
 
-	my_fd->openflags = openflags;
+	my_fd->openflags = FSAL_O_NFS_FLAGS(openflags);
 
 	return fsalstat(ERR_FSAL_NO_ERROR, 0);
 }
@@ -850,15 +918,15 @@ static fsal_status_t ceph_fsal_close(struct fsal_obj_handle *obj_hdl)
 	struct ceph_handle *handle =
 			container_of(obj_hdl, struct ceph_handle, handle);
 
-	if (handle->fd.openflags == FSAL_O_CLOSED)
-		return fsalstat(ERR_FSAL_NOT_OPENED, 0);
-
 	/* Take write lock on object to protect file descriptor.
 	 * This can block over an I/O operation.
 	 */
 	PTHREAD_RWLOCK_wrlock(&obj_hdl->obj_lock);
 
-	status = ceph_close_my_fd(handle, &handle->fd);
+	if (handle->fd.openflags == FSAL_O_CLOSED)
+		status = fsalstat(ERR_FSAL_NOT_OPENED, 0);
+	else
+		status = ceph_close_my_fd(handle, &handle->fd);
 
 	PTHREAD_RWLOCK_unlock(&obj_hdl->obj_lock);
 
@@ -1154,7 +1222,7 @@ static fsal_status_t ceph_fsal_open2(struct fsal_obj_handle *obj_hdl,
 			}
 		} else if (attrs_out && attrs_out->request_mask &
 			   ATTR_RDATTR_ERR) {
-			attrs_out->valid_mask &= ATTR_RDATTR_ERR;
+			attrs_out->valid_mask = ATTR_RDATTR_ERR;
 		}
 
 		if (state == NULL) {
@@ -1218,6 +1286,23 @@ static fsal_status_t ceph_fsal_open2(struct fsal_obj_handle *obj_hdl,
 			return status;
 		}
 
+		if (temp->type != REGULAR_FILE) {
+			if (temp->type == DIRECTORY) {
+				/* Trying to open2 a directory */
+				status = fsalstat(ERR_FSAL_ISDIR, 0);
+			} else {
+				/* Trying to open2 any other non-regular file */
+				status = fsalstat(ERR_FSAL_SYMLINK, 0);
+			}
+
+			/* Release the object we found by lookup. */
+			temp->obj_ops->release(temp);
+			LogFullDebug(COMPONENT_FSAL,
+				     "open2 returning %s",
+				     fsal_err_txt(status));
+			return status;
+		}
+
 		/* Now call ourselves without name and attributes to open. */
 		status = obj_hdl->obj_ops->open2(temp, state, openflags,
 						FSAL_NO_CREATE, NULL, NULL,
@@ -1302,6 +1387,19 @@ static fsal_status_t ceph_fsal_open2(struct fsal_obj_handle *obj_hdl,
 		return ceph2fsal_error(retval);
 	}
 
+	/* Check if the opened file is not a regular file. */
+	if (posix2fsal_type(stx.stx_mode) == DIRECTORY) {
+		/* Trying to open2 a directory */
+		status = fsalstat(ERR_FSAL_ISDIR, 0);
+		goto fileerr;
+	}
+
+	if (posix2fsal_type(stx.stx_mode) != REGULAR_FILE) {
+		/* Trying to open2 any other non-regular file */
+		status = fsalstat(ERR_FSAL_SYMLINK, 0);
+		goto fileerr;
+	}
+
 	/* Remember if we were responsible for creating the file.
 	 * Note that in an UNCHECKED retry we MIGHT have re-created the
 	 * file and won't remember that. Oh well, so in that rare case we
@@ -1336,7 +1434,7 @@ static fsal_status_t ceph_fsal_open2(struct fsal_obj_handle *obj_hdl,
 		my_fd = &hdl->fd;
 
 	my_fd->fd = fd;
-	my_fd->openflags = openflags;
+	my_fd->openflags = FSAL_O_NFS_FLAGS(openflags);
 
 	*new_obj = &hdl->handle;
 
@@ -2045,8 +2143,8 @@ static void ceph_deleg_cb(Fh *fh, void *vhdl)
 	struct ceph_handle *hdl =
 		container_of(obj_hdl, struct ceph_handle, handle);
 	struct gsh_buffdesc key = {
-		.addr = &hdl->vi,
-		.len = sizeof(vinodeno_t)
+		.addr = &hdl->key,
+		.len = sizeof(hdl->key)
 	};
 
 	LogDebug(COMPONENT_FSAL, "Recalling delegations on %p", hdl);
@@ -2290,12 +2388,23 @@ static fsal_status_t ceph_fsal_setattr2(struct fsal_obj_handle *obj_hdl,
 		LogDebug(COMPONENT_FSAL,
 			 "setattrx returned %s (%d)",
 			 strerror(-rc), -rc);
-		status = ceph2fsal_error(rc);
-	} else {
-		/* Success */
-		status = fsalstat(ERR_FSAL_NO_ERROR, 0);
+		goto out;
 	}
 
+	if (FSAL_TEST_MASK(attrib_set->valid_mask, ATTR4_SEC_LABEL)) {
+		rc = fsal_ceph_ll_setxattr(export->cmount, myself->i,
+				export->sec_label_xattr,
+				attrib_set->sec_label.slai_data.slai_data_val,
+				attrib_set->sec_label.slai_data.slai_data_len,
+				0, op_ctx->creds);
+		if (rc < 0) {
+			status = ceph2fsal_error(rc);
+			goto out;
+		}
+	}
+
+	/* Success */
+	status = fsalstat(ERR_FSAL_NO_ERROR, 0);
  out:
 
 	if (has_lock)
@@ -2321,6 +2430,7 @@ static fsal_status_t ceph_fsal_setattr2(struct fsal_obj_handle *obj_hdl,
 static fsal_status_t ceph_fsal_close2(struct fsal_obj_handle *obj_hdl,
 				      struct state_t *state)
 {
+	fsal_status_t status = {0, 0};
 	struct ceph_handle *myself =
 		container_of(obj_hdl, struct ceph_handle, handle);
 	struct ceph_fd *my_fd = &container_of(state, struct ceph_state_fd,
@@ -2341,7 +2451,14 @@ static fsal_status_t ceph_fsal_close2(struct fsal_obj_handle *obj_hdl,
 		PTHREAD_RWLOCK_unlock(&obj_hdl->obj_lock);
 	}
 
-	return ceph_close_my_fd(myself, my_fd);
+	/* Acquire state's fdlock to make sure no other thread
+	 * is operating on the fd while we close it.
+	 */
+	PTHREAD_RWLOCK_wrlock(&my_fd->fdlock);
+	status = ceph_close_my_fd(myself, my_fd);
+	PTHREAD_RWLOCK_unlock(&my_fd->fdlock);
+
+	return status;
 }
 
 /**
@@ -2371,14 +2488,17 @@ ceph_fsal_handle_to_wire(const struct fsal_obj_handle *handle_pub,
 		/* Digested Handles */
 	case FSAL_DIGEST_NFSV3:
 	case FSAL_DIGEST_NFSV4:
-		if (fh_desc->len < sizeof(handle->vi)) {
+		if (fh_desc->len < sizeof(handle->key)) {
 			LogMajor(COMPONENT_FSAL,
 				 "digest_handle: space too small for handle.  Need %zu, have %zu",
-				 sizeof(vinodeno_t), fh_desc->len);
+				 sizeof(handle->key), fh_desc->len);
 			return fsalstat(ERR_FSAL_TOOSMALL, 0);
 		} else {
-			memcpy(fh_desc->addr, &handle->vi, sizeof(vinodeno_t));
-			fh_desc->len = sizeof(handle->vi);
+			if (handle->key.chk_fscid)
+				fh_desc->len = sizeof(handle->key);
+			else
+				fh_desc->len = sizeof(handle->key.chk_vi);
+			memcpy(fh_desc->addr, &handle->key, fh_desc->len);
 		}
 		break;
 
@@ -2406,8 +2526,8 @@ ceph_fsal_handle_to_key(struct fsal_obj_handle *handle_pub,
 	struct ceph_handle *handle =
 		container_of(handle_pub, struct ceph_handle, handle);
 
-	fh_desc->addr = &handle->vi;
-	fh_desc->len = sizeof(vinodeno_t);
+	fh_desc->addr = &handle->key;
+	fh_desc->len = sizeof(handle->key);
 }
 
 #ifdef USE_CEPH_LL_FALLOCATE
