@@ -43,6 +43,7 @@
 #include "fsal_convert.h"
 #include "FSAL/fsal_commonlib.h"
 #include "statx_compat.h"
+#include "nfs_exports.h"
 #include "internal.h"
 
 /**
@@ -69,10 +70,11 @@ void construct_handle(const struct ceph_statx *stx, struct Inode *i,
 
 	constructing = gsh_calloc(1, sizeof(struct ceph_handle));
 
-	constructing->vi.ino.val = stx->stx_ino;
+	constructing->key.chk_vi.ino.val = stx->stx_ino;
 #ifdef CEPH_NOSNAP
-	constructing->vi.snapid.val = stx->stx_dev;
+	constructing->key.chk_vi.snapid.val = stx->stx_dev;
 #endif /* CEPH_NOSNAP */
+	constructing->key.chk_fscid = export->fscid;
 	constructing->i = i;
 	constructing->up_ops = export->export.up_ops;
 
@@ -144,6 +146,10 @@ void ceph2fsal_attributes(const struct ceph_statx *stx,
 	fsalattr->fsid = posix2fsal_fsid(stx->stx_dev);
 	fsalattr->fileid = stx->stx_ino;
 
+	/* Disable seclabels if not enabled in config */
+	if (!op_ctx_export_has_option(EXPORT_OPTION_SECLABEL_SET))
+		fsalattr->supported &= ~ATTR4_SEC_LABEL;
+
 	if (stx->stx_mask & CEPH_STATX_MODE) {
 		fsalattr->valid_mask |= ATTR_MODE;
 		fsalattr->mode = unix2fsal_mode(stx->stx_mode);
