@@ -38,7 +38,8 @@
 #define GLUSTER_VOLPATH_KEY  "volpath"
 
 /* defined the set of attributes supported with POSIX */
-#define GLUSTERFS_SUPPORTED_ATTRIBUTES (ATTRS_POSIX | ATTR_ACL)
+#define GLUSTERFS_SUPPORTED_ATTRIBUTES (ATTRS_POSIX | ATTR_ACL | \
+					ATTR4_SEC_LABEL)
 
 /**
  * The attributes this FSAL can set.
@@ -48,7 +49,7 @@
 ATTR_MODE     | ATTR_OWNER	  | ATTR_GROUP	      |  \
 ATTR_ATIME    | ATTR_CTIME	  | ATTR_MTIME	      |  \
 ATTR_SIZE     | ATTR_MTIME_SERVER | ATTR_ATIME_SERVER |  \
-ATTR_ACL)
+ATTR_ACL      | ATTR4_SEC_LABEL)
 
 /**
  * Override internal Gluster defines for the time being.
@@ -151,6 +152,7 @@ struct glusterfs_export {
 	struct fsal_export export;
 	bool pnfs_ds_enabled;
 	bool pnfs_mds_enabled;
+	char *sec_label_xattr;
 };
 
 struct glusterfs_fd {
@@ -209,12 +211,24 @@ struct glfs_ds_wire {
 typedef struct fsal_xstat__ {
 	int attr_valid;
 	struct stat buffstat;
-	char buffacl[GLFS_ACL_BUF_SIZE];
 	acl_t e_acl; /* stores effective acl */
 	acl_t i_acl; /* stores inherited acl */
 	bool is_dir;
 } glusterfs_fsal_xstat_t;
 
+static inline void glusterfs_fsal_clean_xstat(glusterfs_fsal_xstat_t *buffxstat)
+{
+	if (buffxstat->e_acl) {
+		acl_free(buffxstat->e_acl);
+		buffxstat->e_acl = NULL;
+	}
+
+	if (buffxstat->i_acl) {
+		acl_free(buffxstat->i_acl);
+		buffxstat->i_acl = NULL;
+	}
+}
+
 struct glusterfs_state_fd {
 	struct state_t state;
 	struct glusterfs_fd glusterfs_fd;
