@@ -33,6 +33,7 @@
 #include "pnfs_utils.h"
 #include "nfs_exports.h"
 #include "sal_data.h"
+#include "fsal_types.h"
 
 /* fsal_obj_handle common methods
  */
@@ -124,6 +125,8 @@ static fsal_status_t lookup(struct fsal_obj_handle *parent,
 	    container_of(op_ctx->fsal_export, struct glusterfs_export, export);
 	struct glusterfs_handle *parenthandle =
 	    container_of(parent, struct glusterfs_handle, handle);
+	glusterfs_fsal_xstat_t buffxstat = {.e_acl = NULL, .i_acl = NULL};
+
 #ifdef GLTIMING
 	struct timespec s_time, e_time;
 
@@ -155,6 +158,30 @@ static fsal_status_t lookup(struct fsal_obj_handle *parent,
 
 	if (attrs_out != NULL) {
 		posix2fsal_attributes_all(&sb, attrs_out);
+
+		if (attrs_out->request_mask & ATTR_ACL) {
+			/* Fetch the ACL */
+			status = glusterfs_get_acl(glfs_export,
+						   glhandle,
+						   &buffxstat, attrs_out);
+			if (status.major == ERR_FSAL_NOENT) {
+				if (attrs_out->type == SYMBOLIC_LINK)
+					status = fsalstat(ERR_FSAL_NO_ERROR, 0);
+				else
+					status = gluster2fsal_error(ESTALE);
+			}
+
+			if (!FSAL_IS_ERROR(status)) {
+				/* Success, so mark ACL as valid. */
+				attrs_out->valid_mask |= ATTR_ACL;
+			} else {
+				if (attrs_out->request_mask & ATTR_RDATTR_ERR)
+					attrs_out->valid_mask = ATTR_RDATTR_ERR;
+
+				fsal_release_attrs(attrs_out);
+				goto out;
+			}
+		}
 	}
 
 	*handle = &objhandle->handle;
@@ -167,9 +194,52 @@ static fsal_status_t lookup(struct fsal_obj_handle *parent,
 	latency_update(&s_time, &e_time, lat_lookup);
 #endif
 
+	glusterfs_fsal_clean_xstat(&buffxstat);
 	return status;
 }
 
+static int
+glusterfs_fsal_get_sec_label(struct glusterfs_handle *glhandle,
+			     struct attrlist *attrs)
+{
+	int rc = 0;
+	struct glusterfs_export *export =
+		container_of(op_ctx->fsal_export,
+			     struct glusterfs_export, export);
+
+	if (FSAL_TEST_MASK(attrs->request_mask, ATTR4_SEC_LABEL) &&
+	    op_ctx_export_has_option(EXPORT_OPTION_SECLABEL_SET)) {
+
+		char label[NFS4_OPAQUE_LIMIT];
+
+		rc = glfs_h_getxattrs(export->gl_fs->fs, glhandle->glhandle,
+				export->sec_label_xattr, label,
+				NFS4_OPAQUE_LIMIT);
+
+		if (rc < 0) {
+			/* If there's no label then just do zero-length one */
+			if (errno != ENODATA) {
+				rc = -errno;
+				goto out_err;
+			}
+			rc = 0;
+		}
+
+		attrs->sec_label.slai_data.slai_data_len = rc;
+		gsh_free(attrs->sec_label.slai_data.slai_data_val);
+		if (rc > 0) {
+			attrs->sec_label.slai_data.slai_data_val =
+				gsh_memdup(label, rc);
+			FSAL_SET_MASK(attrs->valid_mask, ATTR4_SEC_LABEL);
+		} else {
+			attrs->sec_label.slai_data.slai_data_val = NULL;
+			FSAL_UNSET_MASK(attrs->valid_mask, ATTR4_SEC_LABEL);
+		}
+	}
+out_err:
+	return rc;
+}
+
 /**
  * @brief Implements GLUSTER FSAL objectoperation readdir
  */
@@ -196,6 +266,7 @@ static fsal_status_t read_dirents(struct fsal_obj_handle *dir_hdl,
 		 (GFAPI_XREADDIRP_STAT | GFAPI_XREADDIRP_HANDLE);
 	struct glfs_object *tmp = NULL;
 	struct stat *sb;
+	glusterfs_fsal_xstat_t buffxstat = {.e_acl = NULL, .i_acl = NULL};
 #endif
 
 #ifdef GLTIMING
@@ -298,11 +369,43 @@ static fsal_status_t read_dirents(struct fsal_obj_handle *dir_hdl,
 			gluster_cleanup_vars(glhandle);
 			goto out;
 		}
+
+		if (attrs.request_mask & ATTR_ACL) {
+			/* Fetch the ACL */
+			status = glusterfs_get_acl(glfs_export,
+						   glhandle,
+						   &buffxstat, &attrs);
+			if (status.major == ERR_FSAL_NOENT) {
+				if (attrs.type == SYMBOLIC_LINK)
+					status = fsalstat(ERR_FSAL_NO_ERROR, 0);
+				else
+					status = gluster2fsal_error(ESTALE);
+			}
+
+			if (!FSAL_IS_ERROR(status)) {
+				/* Success, so mark ACL as valid. */
+				attrs.valid_mask |= ATTR_ACL;
+			} else {
+				if (attrs.request_mask & ATTR_RDATTR_ERR)
+					attrs.valid_mask = ATTR_RDATTR_ERR;
+
+				fsal_release_attrs(&attrs);
+				glusterfs_fsal_clean_xstat(&buffxstat);
+				goto out;
+			}
+		}
 #endif
+		rc = glusterfs_fsal_get_sec_label(objhandle, &attrs);
+		if (rc < 0) {
+			status = gluster2fsal_error(errno);
+			goto out;
+		}
+
 		cb_rc = cb(de.d_name, obj, &attrs,
 			   dir_state, glfs_telldir(glfd));
 
 		fsal_release_attrs(&attrs);
+		glusterfs_fsal_clean_xstat(&buffxstat);
 
 		/* Read ahead not supported by this FSAL. */
 		if (cb_rc >= DIR_READAHEAD)
@@ -777,6 +880,7 @@ static fsal_status_t getattrs(struct fsal_obj_handle *obj_hdl,
 	}
 
 	stat2fsal_attributes(&buffxstat.buffstat, attrs);
+
 	if (obj_hdl->type == DIRECTORY)
 		buffxstat.is_dir = true;
 	else
@@ -792,6 +896,20 @@ static fsal_status_t getattrs(struct fsal_obj_handle *obj_hdl,
 		}
 	}
 
+	rc = glusterfs_fsal_get_sec_label(objhandle, attrs);
+
+	if (rc < 0) {
+		if (errno == ENOENT)
+			status = gluster2fsal_error(ESTALE);
+		else
+			status = gluster2fsal_error(errno);
+		if (attrs->request_mask & ATTR_RDATTR_ERR) {
+			/* Caller asked for error to be visible. */
+			attrs->valid_mask = ATTR_RDATTR_ERR;
+		}
+		goto out;
+	}
+
 	/* *
 	* The error ENOENT is not an expected error for GETATTRS
 	* Due to this, operations such as RENAME will fail when
@@ -818,6 +936,7 @@ static fsal_status_t getattrs(struct fsal_obj_handle *obj_hdl,
 	latency_update(&s_time, &e_time, lat_getattrs);
 #endif
 
+	glusterfs_fsal_clean_xstat(&buffxstat);
 	return status;
 }
 
@@ -1012,7 +1131,7 @@ fsal_status_t glusterfs_open_my_fd(struct glusterfs_handle *objhandle,
 	}
 
 	my_fd->glfd = glfd;
-	my_fd->openflags = openflags;
+	my_fd->openflags = FSAL_O_NFS_FLAGS(openflags);
 	my_fd->creds.caller_uid = op_ctx->creds->caller_uid;
 	my_fd->creds.caller_gid = op_ctx->creds->caller_gid;
 	my_fd->creds.caller_glen = op_ctx->creds->caller_glen;
@@ -1108,16 +1227,17 @@ static fsal_status_t file_close(struct fsal_obj_handle *obj_hdl)
 
 	assert(obj_hdl->type == REGULAR_FILE);
 
-	if (objhandle->globalfd.openflags == FSAL_O_CLOSED)
-		return fsalstat(ERR_FSAL_NOT_OPENED, 0);
-
 	/* Take write lock on object to protect file descriptor.
 	 * This can block over an I/O operation.
 	 */
 	PTHREAD_RWLOCK_wrlock(&obj_hdl->obj_lock);
 
-	status = glusterfs_close_my_fd(&objhandle->globalfd);
-	objhandle->globalfd.openflags = FSAL_O_CLOSED;
+	if (objhandle->globalfd.openflags == FSAL_O_CLOSED) {
+		status = fsalstat(ERR_FSAL_NOT_OPENED, 0);
+	} else {
+		status = glusterfs_close_my_fd(&objhandle->globalfd);
+		objhandle->globalfd.openflags = FSAL_O_CLOSED;
+	}
 
 	PTHREAD_RWLOCK_unlock(&obj_hdl->obj_lock);
 
@@ -1200,6 +1320,10 @@ fsal_status_t find_fd(struct glusterfs_fd *my_fd,
 				glusterfs_close_func,
 				has_lock, closefd, open_for_locks,
 				&reusing_open_state_fd);
+
+	if (FSAL_IS_ERROR(status))
+		goto out;
+
 	/* since tmp2_fd is not accessed/closed outside
 	* this routine, its safe to copy its variables into my_fd
 	* without taking extra reference or allocating extra
@@ -1212,6 +1336,10 @@ fsal_status_t find_fd(struct glusterfs_fd *my_fd,
 				gsh_memdup(tmp2_fd->creds.caller_garray,
 					   tmp2_fd->creds.caller_glen *
 					   sizeof(gid_t));
+		/* Since we dup the fd, we need to close it post
+		 * processing the fop.
+		 */
+		*closefd = true;
 	} else {
 		my_fd->glfd = tmp2_fd->glfd;
 		my_fd->creds.caller_garray = tmp2_fd->creds.caller_garray;
@@ -1222,6 +1350,7 @@ fsal_status_t find_fd(struct glusterfs_fd *my_fd,
 	my_fd->creds.caller_gid = tmp2_fd->creds.caller_gid;
 	my_fd->creds.caller_glen = tmp2_fd->creds.caller_glen;
 
+out:
 	return status;
 }
 
@@ -1438,7 +1567,7 @@ static fsal_status_t glusterfs_open2(struct fsal_obj_handle *obj_hdl,
 
 		} else if (attrs_out && attrs_out->request_mask &
 			   ATTR_RDATTR_ERR) {
-			attrs_out->valid_mask &= ATTR_RDATTR_ERR;
+			attrs_out->valid_mask = ATTR_RDATTR_ERR;
 		}
 
 		if (state == NULL) {
@@ -1536,6 +1665,22 @@ static fsal_status_t glusterfs_open2(struct fsal_obj_handle *obj_hdl,
 			goto direrr;
 		}
 
+		if ((*new_obj)->type != REGULAR_FILE) {
+			if ((*new_obj)->type == DIRECTORY) {
+				/* Trying to open2 a directory */
+				status = fsalstat(ERR_FSAL_ISDIR, 0);
+			} else {
+				/* Trying to open2 any other non-regular file */
+				status = fsalstat(ERR_FSAL_SYMLINK, 0);
+			}
+
+			/* Release the object we found by lookup. */
+			LogFullDebug(COMPONENT_FSAL,
+				     "open2 returning %s",
+				     fsal_err_txt(status));
+			goto direrr;
+		}
+
 		myself = container_of(*new_obj,
 				      struct glusterfs_handle,
 				      handle);
@@ -1590,6 +1735,19 @@ static fsal_status_t glusterfs_open2(struct fsal_obj_handle *obj_hdl,
 		goto out;
 	}
 
+	/* Check if the opened file is not a regular file. */
+	if (posix2fsal_type(sb.st_mode) == DIRECTORY) {
+		/* Trying to open2 a directory */
+		status = fsalstat(ERR_FSAL_ISDIR, 0);
+		goto direrr;
+	}
+
+	if (posix2fsal_type(sb.st_mode) != REGULAR_FILE) {
+		/* Trying to open2 any other non-regular file */
+		status = fsalstat(ERR_FSAL_SYMLINK, 0);
+		goto direrr;
+	}
+
 	/* Remember if we were responsible for creating the file.
 	 * Note that in an UNCHECKED retry we MIGHT have re-created the
 	 * file and won't remember that. Oh well, so in that rare case we
@@ -1816,6 +1974,14 @@ static fsal_status_t glusterfs_reopen2(struct fsal_obj_handle *obj_hdl,
 		glusterfs_close_my_fd(my_share_fd);
 		my_share_fd->glfd = my_fd->glfd;
 		my_share_fd->openflags = my_fd->openflags;
+		my_share_fd->creds.caller_uid = my_fd->creds.caller_uid;
+		my_share_fd->creds.caller_gid = my_fd->creds.caller_gid;
+		my_share_fd->creds.caller_glen = my_fd->creds.caller_glen;
+		my_share_fd->creds.caller_garray = my_fd->creds.caller_garray;
+#ifdef USE_GLUSTER_DELEGATION
+		memcpy(my_share_fd->lease_id, my_fd->lease_id,
+		       GLAPI_LEASE_ID_SIZE);
+#endif
 
 		PTHREAD_RWLOCK_unlock(&my_share_fd->fdlock);
 	} else {
@@ -2309,7 +2475,7 @@ static fsal_status_t glusterfs_lock_op2(struct fsal_obj_handle *obj_hdl,
 
 	if (FSAL_IS_ERROR(status)) {
 		LogCrit(COMPONENT_FSAL, "Unable to find fd for lock operation");
-		return status;
+		goto err;
 	}
 
 	errno = 0;
@@ -2394,7 +2560,10 @@ static fsal_status_t glusterfs_lock_op2(struct fsal_obj_handle *obj_hdl,
 	if (has_lock)
 		PTHREAD_RWLOCK_unlock(&obj_hdl->obj_lock);
 
-	return fsalstat(posix2fsal_error(retval), retval);
+	if (retval)
+		status = gluster2fsal_error(retval);
+
+	return status;
 }
 
 /**
@@ -2425,7 +2594,7 @@ static fsal_status_t glusterfs_setattr2(struct fsal_obj_handle *obj_hdl,
 	struct glusterfs_fd my_fd = {0};
 	struct glusterfs_export *glfs_export =
 	    container_of(op_ctx->fsal_export, struct glusterfs_export, export);
-	glusterfs_fsal_xstat_t buffxstat;
+	glusterfs_fsal_xstat_t buffxstat = { 0 };
 	int attr_valid = 0;
 	int mask = 0;
 	struct glusterfs_fd *glusterfs_fd = NULL;
@@ -2554,7 +2723,8 @@ static fsal_status_t glusterfs_setattr2(struct fsal_obj_handle *obj_hdl,
 	/* EATTRNOTSUPP error.  */
 
 	if (NFSv4_ACL_SUPPORT) {
-		if (FSAL_TEST_MASK(attrib_set->valid_mask, ATTR_ACL)) {
+		if (FSAL_TEST_MASK(attrib_set->valid_mask, ATTR_ACL) &&
+		    attrib_set->acl && attrib_set->acl->naces) {
 			if (obj_hdl->type == DIRECTORY)
 				buffxstat.is_dir = true;
 			else
@@ -2607,6 +2777,23 @@ static fsal_status_t glusterfs_setattr2(struct fsal_obj_handle *obj_hdl,
 	if (FSAL_IS_ERROR(status)) {
 		LogDebug(COMPONENT_FSAL,
 			 "setting ACL failed");
+		goto creds;
+	}
+
+	if (FSAL_TEST_MASK(attrib_set->valid_mask, ATTR4_SEC_LABEL) &&
+			op_ctx_export_has_option(EXPORT_OPTION_SECLABEL_SET)) {
+		retval = glfs_h_setxattrs(glfs_export->gl_fs->fs,
+				myself->glhandle,
+				glfs_export->sec_label_xattr,
+				attrib_set->sec_label.slai_data.slai_data_val,
+				attrib_set->sec_label.slai_data.slai_data_len,
+				0);
+		if (retval < 0) {
+			status = gluster2fsal_error(errno);
+			LogCrit(COMPONENT_FSAL,
+					"Error : seclabel failed with error %s",
+					strerror(errno));
+		}
 	}
 
 creds:
@@ -2628,6 +2815,7 @@ out:
 	if (has_lock)
 		PTHREAD_RWLOCK_unlock(&obj_hdl->obj_lock);
 
+	glusterfs_fsal_clean_xstat(&buffxstat);
 	return status;
 }
 
@@ -2638,6 +2826,7 @@ static fsal_status_t glusterfs_close2(struct fsal_obj_handle *obj_hdl,
 				      struct state_t *state)
 {
 	struct glusterfs_handle *myself = NULL;
+	fsal_status_t status = {0, 0};
 	struct glusterfs_fd *my_fd = &container_of(state,
 						   struct glusterfs_state_fd,
 						   state)->glusterfs_fd;
@@ -2661,7 +2850,14 @@ static fsal_status_t glusterfs_close2(struct fsal_obj_handle *obj_hdl,
 		PTHREAD_RWLOCK_unlock(&obj_hdl->obj_lock);
 	}
 
-	return glusterfs_close_my_fd(my_fd);
+	/* Acquire state's fdlock to make sure no other thread
+	 * is operating on the fd while we close it.
+	 */
+	PTHREAD_RWLOCK_wrlock(&my_fd->fdlock);
+	status = glusterfs_close_my_fd(my_fd);
+	PTHREAD_RWLOCK_unlock(&my_fd->fdlock);
+
+	return status;
 }
 
 /**
