@@ -60,7 +60,7 @@ gpfs_open_func(struct fsal_obj_handle *obj_hdl, fsal_openflags_t openflags,
 	if (FSAL_IS_ERROR(status))
 		return status;
 
-	my_fd->openflags = openflags;
+	my_fd->openflags = FSAL_O_NFS_FLAGS(openflags);
 	LogFullDebug(COMPONENT_FSAL, "new fd %d", my_fd->fd);
 
 	return status;
@@ -191,8 +191,9 @@ open_by_handle(struct fsal_obj_handle *obj_hdl, struct state_t *state,
 		/* assert(state == NULL); */
 		(void)fsal_internal_close(my_fd->fd, NULL, 0);
 	}
+
 	my_fd->fd = fd;
-	my_fd->openflags = openflags;
+	my_fd->openflags = FSAL_O_NFS_FLAGS(openflags);
 
 	if (attrs_out && (createmode >= FSAL_EXCLUSIVE || truncated)) {
 		/* Refresh the attributes */
@@ -211,7 +212,7 @@ open_by_handle(struct fsal_obj_handle *obj_hdl, struct state_t *state,
 						  EEXIST);
 		}
 	} else if (attrs_out && attrs_out->request_mask & ATTR_RDATTR_ERR) {
-		attrs_out->valid_mask &= ATTR_RDATTR_ERR;
+		attrs_out->valid_mask = ATTR_RDATTR_ERR;
 	}
 
 	if (state == NULL) {
@@ -267,6 +268,23 @@ open_by_name(struct fsal_obj_handle *obj_hdl, struct state_t *state,
 		return status;
 	}
 
+	if (temp->type != REGULAR_FILE) {
+		if (temp->type == DIRECTORY) {
+			/* Trying to open2 a directory */
+			status = fsalstat(ERR_FSAL_ISDIR, 0);
+		} else {
+			/* Trying to open2 any other non-regular file */
+			status = fsalstat(ERR_FSAL_SYMLINK, 0);
+		}
+
+		/* Release the object we found by lookup. */
+		temp->obj_ops->release(temp);
+		LogFullDebug(COMPONENT_FSAL,
+			     "open returned %s",
+			     fsal_err_txt(status));
+		return status;
+	}
+
 	status = open_by_handle(temp, state, openflags, posix_flags,
 				verifier, attrs_out, FSAL_NO_CREATE, cpm_check);
 
@@ -370,6 +388,12 @@ gpfs_open2(struct fsal_obj_handle *obj_hdl, struct state_t *state,
 				    posix_flags, verifier, attrs_out,
 				    caller_perm_check);
 
+	/** @todo: to proceed past here, we need a struct attrlist in order to
+	 *         create the fsal_obj_handle, so if it actually is NULL (it
+	 *         will actually never be since mdcache will always ask for
+	 *         attributes) we really should create a temporary attrlist...
+	 */
+
 	posix_flags |= O_CREAT;
 
 	/* And if we are at least FSAL_GUARDED, do an O_EXCL create. */
@@ -429,6 +453,22 @@ gpfs_open2(struct fsal_obj_handle *obj_hdl, struct state_t *state,
 	created = (posix_flags & O_EXCL) != 0;
 	*caller_perm_check = false;
 
+	/* Check if the object type is SYMBOLIC_LINK for a state object.
+	 * If yes, then give error ERR_FSAL_SYMLINK.
+	 */
+	if (state != NULL &&
+		attrs_out != NULL && attrs_out->type != REGULAR_FILE) {
+		LogDebug(COMPONENT_FSAL, "Trying to open a non-regular file");
+			if (attrs_out->type == DIRECTORY) {
+				/* Trying to open2 a directory */
+				status = fsalstat(ERR_FSAL_ISDIR, 0);
+			} else {
+				/* Trying to open2 any other non-regular file */
+				status = fsalstat(ERR_FSAL_SYMLINK, 0);
+			}
+		goto fileerr;
+	}
+
 	/* allocate an obj_handle and fill it up */
 	hdl = alloc_handle(&fh, obj_hdl->fs, attrs_out, NULL, export);
 	if (hdl == NULL) {
@@ -640,8 +680,9 @@ gpfs_reopen2(struct fsal_obj_handle *obj_hdl, struct state_t *state,
 		PTHREAD_RWLOCK_wrlock(&my_share_fd->fdlock);
 
 		fsal_internal_close(my_share_fd->fd, NULL, 0);
+
 		my_share_fd->fd = my_fd;
-		my_share_fd->openflags = openflags;
+		my_share_fd->openflags = FSAL_O_NFS_FLAGS(openflags);
 
 		PTHREAD_RWLOCK_unlock(&my_share_fd->fdlock);
 	} else {
@@ -1461,9 +1502,15 @@ gpfs_close2(struct fsal_obj_handle *obj_hdl, struct state_t *state)
 			     "state %p fd %d", state, my_fd->fd);
 		state_owner = state->state_owner;
 
+		/* Acquire state's fdlock to make sure no other thread
+		 * is operating on the fd while we close it.
+		 */
+		PTHREAD_RWLOCK_wrlock(&my_fd->fdlock);
 		status = fsal_internal_close(my_fd->fd, state_owner, 0);
+
 		my_fd->fd = -1;
 		my_fd->openflags = FSAL_O_CLOSED;
+		PTHREAD_RWLOCK_unlock(&my_fd->fdlock);
 	}
 	if (FSAL_IS_ERROR(status)) {
 		struct gpfs_file_handle *gfh = container_of(obj_hdl,
