@@ -502,9 +502,24 @@ static fsal_status_t read_dirents(struct fsal_obj_handle *dir_hdl,
 			fsal_prepare_attrs(&attrs, attrmask);
 
 			status = lookup(dir_hdl, dentry->d_name, &hdl, &attrs);
+
+			/* The entries we get in getdents64 syscall may
+			 * not be there by the time we do lookup, so
+			 * handle some errors here.
+			 *
+			 * Since we do lookup by name, we mostly get
+			 * ERR_FSAL_NOENT, but handle other similar
+			 * errors!
+			 */
 			if (FSAL_IS_ERROR(status)) {
-				fsal_error = status.major;
-				goto done;
+				if (status.major == ERR_FSAL_NOENT ||
+				    status.major == ERR_FSAL_STALE ||
+				    status.major == ERR_FSAL_XDEV) {
+					goto skip;
+				} else {
+					fsal_error = status.major;
+					goto done;
+				}
 			}
 
 			/* callback to cache inode */
@@ -571,13 +586,20 @@ static fsal_status_t getattrs(struct fsal_obj_handle *obj_hdl,
 		goto out;
 	}
 
-	status = GPFSFSAL_fs_loc(op_ctx->fsal_export,
-				 obj_hdl->fs->private_data,
-				 op_ctx, myself->handle,
-				 attrs);
+	if (obj_hdl->type != DIRECTORY)
+		goto out;
 
-	if (FSAL_IS_SUCCESS(status)) {
+	fsal_status_t fs_loc_status = GPFSFSAL_fs_loc(op_ctx->fsal_export,
+						      obj_hdl->fs->private_data,
+						      op_ctx, myself->handle,
+						      attrs);
+
+	if (FSAL_IS_SUCCESS(fs_loc_status)) {
 		FSAL_SET_MASK(attrs->valid_mask, ATTR4_FS_LOCATIONS);
+	} else {
+		LogDebug(COMPONENT_FSAL,
+			 "Request for attribute fs_locations failed with error: %s",
+			 msg_fsal_err(fs_loc_status.major));
 	}
 
 out:
