@@ -886,7 +886,7 @@ fsal_status_t rgw_fsal_open2(struct fsal_obj_handle *obj_hdl,
 
 		} else if (attrs_out && attrs_out->request_mask &
 			   ATTR_RDATTR_ERR) {
-			attrs_out->valid_mask &= ATTR_RDATTR_ERR;
+			attrs_out->valid_mask = ATTR_RDATTR_ERR;
 		}
 
 		if (!state) {
@@ -952,6 +952,23 @@ fsal_status_t rgw_fsal_open2(struct fsal_obj_handle *obj_hdl,
 			return status;
 		}
 
+		if (temp->type != REGULAR_FILE) {
+			if (temp->type == DIRECTORY) {
+				/* Trying to open2 a directory */
+				status = fsalstat(ERR_FSAL_ISDIR, 0);
+			} else {
+				/* Trying to open2 any other non-regular file */
+				status = fsalstat(ERR_FSAL_SYMLINK, 0);
+			}
+
+			/* Release the object we found by lookup. */
+			temp->obj_ops->release(temp);
+			LogFullDebug(COMPONENT_FSAL,
+				     "open2 returning %s",
+				     fsal_err_txt(status));
+			return status;
+		}
+
 		/* Now call ourselves without name and attributes to open. */
 		status = obj_hdl->obj_ops->open2(temp, state, openflags,
 						FSAL_NO_CREATE, NULL, NULL,
@@ -1045,6 +1062,19 @@ fsal_status_t rgw_fsal_open2(struct fsal_obj_handle *obj_hdl,
 
 	construct_handle(export, rgw_fh, &st, &obj);
 
+	/* Check if the opened file is not a regular file. */
+	if (posix2fsal_type(st.st_mode) == DIRECTORY) {
+		/* Trying to open2 a directory */
+		status = fsalstat(ERR_FSAL_ISDIR, 0);
+		goto fileerr;
+	}
+
+	if (posix2fsal_type(st.st_mode) != REGULAR_FILE) {
+		/* Trying to open2 any other non-regular file */
+		status = fsalstat(ERR_FSAL_SYMLINK, 0);
+		goto fileerr;
+	}
+
 	/* here FSAL_CEPH operates on its (for RGW non-existent) global
 	 * fd */
 #if 0
@@ -1059,7 +1089,7 @@ fsal_status_t rgw_fsal_open2(struct fsal_obj_handle *obj_hdl,
 
 	my_fd->fd = fd;
 #endif
-	handle->openflags = openflags;
+	handle->openflags = FSAL_O_NFS_FLAGS(openflags);
 
 	*new_obj = &obj->handle;
 
@@ -1488,6 +1518,7 @@ fsal_status_t rgw_fsal_close2(struct fsal_obj_handle *obj_hdl,
 {
 	int rc;
 	struct rgw_open_state *open_state;
+	fsal_status_t status;
 
 	struct rgw_export *export =
 	    container_of(op_ctx->fsal_export, struct rgw_export, export);
@@ -1498,6 +1529,8 @@ fsal_status_t rgw_fsal_close2(struct fsal_obj_handle *obj_hdl,
 	LogFullDebug(COMPONENT_FSAL,
 		"%s enter obj_hdl %p state %p", __func__, obj_hdl, state);
 
+	PTHREAD_RWLOCK_wrlock(&obj_hdl->obj_lock);
+
 	if (state) {
 		open_state = (struct rgw_open_state *) state;
 
@@ -1507,28 +1540,32 @@ fsal_status_t rgw_fsal_close2(struct fsal_obj_handle *obj_hdl,
 		if (state->state_type == STATE_TYPE_SHARE ||
 			state->state_type == STATE_TYPE_NLM_SHARE ||
 			state->state_type == STATE_TYPE_9P_FID) {
+
 			/* This is a share state, we must update the share
 			 * counters.  This can block over an I/O operation.
 			 */
-			PTHREAD_RWLOCK_wrlock(&obj_hdl->obj_lock);
 
 			update_share_counters(&handle->share,
 					handle->openflags,
 					FSAL_O_CLOSED);
 
-			PTHREAD_RWLOCK_unlock(&obj_hdl->obj_lock);
 		}
 	} else if (handle->openflags == FSAL_O_CLOSED) {
-		return fsalstat(ERR_FSAL_NOT_OPENED, 0);
+		status = fsalstat(ERR_FSAL_NOT_OPENED, 0);
+	} else {
+		rc = rgw_close(export->rgw_fs, handle->rgw_fh,
+			       RGW_CLOSE_FLAG_NONE);
+		if (rc < 0) {
+			status = rgw2fsal_error(rc);
+		} else {
+			handle->openflags = FSAL_O_CLOSED;
+			status = fsalstat(ERR_FSAL_NO_ERROR, 0);
+		}
 	}
 
-	rc = rgw_close(export->rgw_fs, handle->rgw_fh, RGW_CLOSE_FLAG_NONE);
-	if (rc < 0)
-		return rgw2fsal_error(rc);
+	PTHREAD_RWLOCK_unlock(&obj_hdl->obj_lock);
 
-	handle->openflags = FSAL_O_CLOSED;
-
-	return fsalstat(ERR_FSAL_NO_ERROR, 0);
+	return status;
 }
 
 /**
