@@ -79,7 +79,7 @@ fsal_status_t vfs_open_my_fd(struct vfs_fsal_obj_handle *myself,
 				"fd = %d, new openflags = %x",
 				fd, openflags);
 		my_fd->fd = fd;
-		my_fd->openflags = openflags;
+		my_fd->openflags = FSAL_O_NFS_FLAGS(openflags);
 	}
 
 	return fsalstat(fsal_error, retval);
@@ -163,15 +163,15 @@ fsal_status_t vfs_close(struct fsal_obj_handle *obj_hdl)
 		return fsalstat(posix2fsal_error(EXDEV), EXDEV);
 	}
 
-	if (myself->u.file.fd.openflags == FSAL_O_CLOSED)
-		return fsalstat(ERR_FSAL_NOT_OPENED, 0);
-
 	/* Take write lock on object to protect file descriptor.
 	 * This can block over an I/O operation.
 	 */
 	PTHREAD_RWLOCK_wrlock(&obj_hdl->obj_lock);
 
-	status = vfs_close_my_fd(&myself->u.file.fd);
+	if (myself->u.file.fd.openflags == FSAL_O_CLOSED)
+		status = fsalstat(ERR_FSAL_NOT_OPENED, 0);
+	else
+		status = vfs_close_my_fd(&myself->u.file.fd);
 
 	PTHREAD_RWLOCK_unlock(&obj_hdl->obj_lock);
 
@@ -494,7 +494,7 @@ static fsal_status_t vfs_open2_by_handle(struct fsal_obj_handle *obj_hdl,
 
 		fsal_release_attrs(&attrs);
 	} else if (attrs_out && attrs_out->request_mask & ATTR_RDATTR_ERR) {
-		attrs_out->valid_mask &= ATTR_RDATTR_ERR;
+		attrs_out->valid_mask = ATTR_RDATTR_ERR;
 	}
 
 	if (state == NULL) {
@@ -850,6 +850,19 @@ fsal_status_t vfs_open2(struct fsal_obj_handle *obj_hdl,
 		goto fileerr;
 	}
 
+	/* Check if the opened file is not a regular file. */
+	if (posix2fsal_type(stat.st_mode) == DIRECTORY) {
+		/* Trying to open2 a directory */
+		status = fsalstat(ERR_FSAL_ISDIR, 0);
+		goto fileerr;
+	}
+
+	if (posix2fsal_type(stat.st_mode) != REGULAR_FILE) {
+		/* Trying to open2 any other non-regular file */
+		status = fsalstat(ERR_FSAL_SYMLINK, 0);
+		goto fileerr;
+	}
+
 	/* allocate an obj_handle and fill it up */
 	hdl = alloc_handle(dir_fd, fh, obj_hdl->fs, &stat, myself->handle, name,
 			   op_ctx->fsal_export);
@@ -871,7 +884,7 @@ fsal_status_t vfs_open2(struct fsal_obj_handle *obj_hdl,
 	}
 
 	my_fd->fd = fd;
-	my_fd->openflags = openflags;
+	my_fd->openflags = FSAL_O_NFS_FLAGS(openflags);
 
 	*new_obj = &hdl->obj_handle;
 
@@ -2055,6 +2068,7 @@ fsal_status_t vfs_close2(struct fsal_obj_handle *obj_hdl,
 			 struct state_t *state)
 {
 	struct vfs_fsal_obj_handle *myself = NULL;
+	fsal_status_t status = {0, 0};
 	struct vfs_fd *my_fd = &container_of(state, struct vfs_state_fd,
 					     state)->vfs_fd;
 
@@ -2077,5 +2091,12 @@ fsal_status_t vfs_close2(struct fsal_obj_handle *obj_hdl,
 		PTHREAD_RWLOCK_unlock(&obj_hdl->obj_lock);
 	}
 
-	return vfs_close_my_fd(my_fd);
+	/* Acquire state's fdlock to make sure no other thread
+	 * is operating on the fd while we close it.
+	 */
+	PTHREAD_RWLOCK_wrlock(&my_fd->fdlock);
+	status = vfs_close_my_fd(my_fd);
+	PTHREAD_RWLOCK_unlock(&my_fd->fdlock);
+
+	return status;
 }
