@@ -1,7 +1,7 @@
 /*
  * vim:noexpandtab:shiftwidth=8:tabstop=8:
  *
- * Copyright 2015-2018 Red Hat, Inc. and/or its affiliates.
+ * Copyright 2015-2019 Red Hat, Inc. and/or its affiliates.
  * Author: Daniel Gryniewicz <dang@redhat.com>
  *
  * This program is free software; you can redistribute it and/or
@@ -763,13 +763,13 @@ unlock:
 out:
 	/* Refresh, if necessary.  Must be done without lock held */
 	if (FSAL_IS_SUCCESS(status)) {
-		/* If we're moving a directory out, update parent hash */
+		/* If we're moving a directory out, update parent hash.
+		 * Since we already dropped the src_desk lock, things
+		 * may have changed, so just free the parent fh.
+		 */
 		if (mdc_olddir != mdc_newdir && obj_hdl->type == DIRECTORY) {
 			PTHREAD_RWLOCK_wrlock(&mdc_obj->content_lock);
-
 			mdcache_free_fh(&mdc_obj->fsobj.fsdir.parent);
-			mdc_dir_add_parent(mdc_obj, mdc_newdir);
-
 			PTHREAD_RWLOCK_unlock(&mdc_obj->content_lock);
 		}
 
@@ -828,6 +828,10 @@ fsal_status_t mdcache_refresh_attrs(mdcache_entry_t *entry, bool need_acl,
 
 	/* We will want all the requested attributes in the entry */
 	entry->attrs.request_mask = attrs.request_mask;
+	if (entry->attrs.acl != NULL) {
+		/* request_mask & ATTR_ACL must match attrs.acl */
+		entry->attrs.request_mask |= ATTR_ACL;
+	}
 
 	subcall(
 		status = entry->sub_handle->obj_ops->getattrs(
@@ -978,7 +982,8 @@ static fsal_status_t mdcache_setattr2(struct fsal_obj_handle *obj_hdl,
 		/* Assume that the cache is bogus now */
 		atomic_clear_uint32_t_bits(&entry->mde_flags,
 				MDCACHE_TRUST_ATTRS | MDCACHE_TRUST_ACL |
-				MDCACHE_TRUST_FS_LOCATIONS);
+				MDCACHE_TRUST_FS_LOCATIONS |
+				MDCACHE_TRUST_SEC_LABEL);
 		if (status2.major == ERR_FSAL_STALE)
 			kill_entry = true;
 	} else if (change == entry->attrs.change) {
@@ -1531,9 +1536,7 @@ fsal_status_t mdcache_lookup_path(struct fsal_export *exp_hdl,
 			     "lookup_path Created entry %p FSAL %s",
 			     new_entry, new_entry->sub_handle->fsal->name);
 		/* Make sure this entry has a parent pointer */
-		PTHREAD_RWLOCK_wrlock(&new_entry->content_lock);
-		mdc_get_parent(export, new_entry);
-		PTHREAD_RWLOCK_unlock(&new_entry->content_lock);
+		mdc_get_parent(export, new_entry, NULL);
 
 		*handle = &new_entry->obj_handle;
 	}
@@ -1575,9 +1578,7 @@ fsal_status_t mdcache_create_handle(struct fsal_export *exp_hdl,
 		return status;
 
 	/* Make sure this entry has a parent pointer */
-	PTHREAD_RWLOCK_wrlock(&entry->content_lock);
-	mdc_get_parent(export, entry);
-	PTHREAD_RWLOCK_unlock(&entry->content_lock);
+	mdc_get_parent(export, entry, NULL);
 
 	if (attrs_out != NULL) {
 		LogAttrlist(COMPONENT_CACHE_INODE, NIV_FULL_DEBUG,
