@@ -1,7 +1,7 @@
 /*
  * vim:noexpandtab:shiftwidth=8:tabstop=8:
  *
- * Copyright 2015-2018 Red Hat, Inc. and/or its affiliates.
+ * Copyright 2015-2019 Red Hat, Inc. and/or its affiliates.
  * Author: Daniel Gryniewicz <dang@redhat.com>
  *
  * This program is free software; you can redistribute it and/or
@@ -60,6 +60,30 @@ typedef enum {
 	MDC_REASON_SCAN		/**< Is being inserted by a scan */
 } mdc_reason_t;
 
+typedef struct mdcache_dmap_entry__ {
+	/** AVL node in tree by cookie */
+	struct avltree_node node;
+	/** Entry in LRU */
+	struct glist_head lru_entry;
+	/** Cookie */
+	uint64_t ck;
+	/** Name */
+	char *name;
+	/** Timestamp on entry */
+	struct timespec timestamp;
+} mdcache_dmap_entry_t;
+
+typedef struct {
+	/** Lock protecting this structure */
+	pthread_mutex_t mtx;
+	/** Mapping of ck -> name for whence-is-name */
+	struct avltree map;
+	/** LRU of dirent map entries */
+	struct glist_head lru;
+	/** Count of entries in LRU */
+	uint32_t count;
+} mdc_dirmap_t;
+
 /*
  * MDCACHE internal export
  */
@@ -76,6 +100,10 @@ struct mdcache_fsal_export {
 	pthread_rwlock_t mdc_exp_lock;
 	/** Flags for the export. */
 	uint8_t flags;
+	/** Mapping of ck -> name for whence-is-name */
+	mdc_dirmap_t dirent_map;
+	/** Thread for dirmap processing */
+	struct fridgethr *dirmap_fridge;
 };
 
 /**
@@ -194,8 +222,10 @@ struct entry_export_map {
 #define MDCACHE_DIR_POPULATED FSAL_UP_INVALIDATE_DIR_POPULATED
 /** The directory chunks are considered valid */
 #define MDCACHE_TRUST_DIR_CHUNKS FSAL_UP_INVALIDATE_DIR_CHUNKS
-/** The fs locaitons are considered calid */
+/** The fs_locations are considered valid */
 #define MDCACHE_TRUST_FS_LOCATIONS FSAL_UP_INVALIDATE_FS_LOCATIONS
+/** The sec_labels are considered valid */
+#define MDCACHE_TRUST_SEC_LABEL FSAL_UP_INVALIDATE_SEC_LABEL
 /** The entry has been removed, but not unhashed due to state */
 static const uint32_t MDCACHE_UNREACHABLE = 0x100;
 
@@ -308,6 +338,10 @@ struct mdcache_fsal_obj_handle {
 			struct state_hdl dhdl; /**< Storage for dir state */
 			/** The parent host-handle of this directory ('..') */
 			struct gsh_buffdesc parent;
+			/** Time at which we last refreshed parent
+			  * host-handle.
+			  */
+			time_t parent_time;
 			/** The first dirent cookie in this directory.
 			 *  0 if not known.
 			 */
@@ -345,10 +379,6 @@ struct dir_chunk {
 	int num_entries;
 };
 
-#define mdc_prev_chunk(c) glist_prev_entry(&(c)->parent->fsobj.fsdir.chunks, \
-			struct dir_chunk, chunks, &(c)->chunks)
-
-
 /**
  * @brief Represents a cached directory entry
  *
@@ -358,7 +388,6 @@ struct dir_chunk {
 
 #define DIR_ENTRY_FLAG_NONE     0x0000
 #define DIR_ENTRY_FLAG_DELETED  0x0001
-#define DIR_ENTRY_REFFED        0x0002
 #define DIR_ENTRY_SORTED        0x0004
 
 typedef struct mdcache_dir_entry__ {
@@ -384,8 +413,13 @@ typedef struct mdcache_dir_entry__ {
 	uint64_t namehash;
 	/** Key of cache entry */
 	mdcache_key_t ckey;
-	/** Flags */
+	/** Flags
+	 * Protected by write content_lock or atomics. */
 	uint32_t flags;
+	/** Temporary entry pointer
+	 * Only valid while the entry is ref'd.  Must be NULL otherwise.
+	 * Protected by the parent content_lock */
+	mdcache_entry_t *entry;
 	const char *name;
 	/** The NUL-terminated filename */
 	char name_buffer[];
@@ -498,11 +532,14 @@ fsal_status_t mdcache_readdir_chunked(mdcache_entry_t *directory,
 				      attrmask_t attrmask,
 				      bool *eod_met);
 
-void mdc_get_parent(struct mdcache_fsal_export *exp,
-		    mdcache_entry_t *entry);
+fsal_status_t mdc_get_parent(struct mdcache_fsal_export *exp,
+		    mdcache_entry_t *entry,
+		    struct gsh_buffdesc *parent_out);
 
 void mdc_update_attr_cache(mdcache_entry_t *entry, struct attrlist *attrs);
 
+static inline void mdcache_free_fh(struct gsh_buffdesc *fh_desc);
+
 /**
  * @brief Atomically test the bits in mde_flags.
  *
@@ -618,6 +655,24 @@ mdcache_key_dup(mdcache_key_t *tgt,
 	tgt->fsal = src->fsal;
 }
 
+/**
+ * @brief Check if the parent key of an entry has expired.
+ *
+ * If the parent key is valid return true else return false.
+ *
+ * @param[in] entry     Entry whose parent key may have expired.
+ * @return Return true if valid, false if invalid.
+ */
+static inline bool
+mdcache_is_parent_valid(mdcache_entry_t *entry)
+{
+	time_t current_time = time(NULL);
+
+	if (current_time > entry->fsobj.fsdir.parent_time)
+		return false;
+	return true;
+}
+
 /**
  * @brief Set the parent key of an entry
  *
@@ -629,13 +684,22 @@ mdcache_key_dup(mdcache_key_t *tgt,
 static inline void
 mdc_dir_add_parent(mdcache_entry_t *entry, mdcache_entry_t *mdc_parent)
 {
-	if (entry->fsobj.fsdir.parent.len == 0) {
-		/* The parent key must be a host-handle so that
-		 * create_handle() works in all cases.
-		 */
-		mdc_get_parent_handle(mdc_cur_export(), entry,
-				      mdc_parent->sub_handle);
+	if (entry->fsobj.fsdir.parent.len != 0) {
+		/* Already has a parent pointer */
+		if (entry->fsobj.fsdir.parent_time == 0 ||
+		    mdcache_is_parent_valid(entry)) {
+			return;
+		} else {
+			/* Clean up parent key */
+			mdcache_free_fh(&entry->fsobj.fsdir.parent);
+		}
 	}
+
+	/* The parent key must be a host-handle so that
+	 * create_handle() works in all cases.
+	 */
+	mdc_get_parent_handle(mdc_cur_export(), entry,
+			      mdc_parent->sub_handle);
 }
 
 /**
@@ -704,7 +768,9 @@ mdc_fixup_md(mdcache_entry_t *entry, struct attrlist *attrs)
 	 * attributes. Note that if not all could be provided, we assumed
 	 * that an error occurred.
 	 */
-	if (attrs->request_mask & ~(ATTR_ACL|ATTR4_FS_LOCATIONS))
+	if (attrs->request_mask & ~(ATTR_ACL |
+				    ATTR4_FS_LOCATIONS |
+				    ATTR4_SEC_LABEL))
 		flags |= MDCACHE_TRUST_ATTRS;
 
 	if (attrs->valid_mask == ATTR_RDATTR_ERR) {
@@ -722,6 +788,11 @@ mdc_fixup_md(mdcache_entry_t *entry, struct attrlist *attrs)
 		flags |= MDCACHE_TRUST_FS_LOCATIONS;
 	}
 
+	if (attrs->request_mask & ATTR4_SEC_LABEL &&
+		attrs->sec_label.slai_data.slai_data_val != NULL) {
+		flags |= MDCACHE_TRUST_SEC_LABEL;
+	}
+
 	time_t cur_time = time(NULL);
 
 	/* Set the refresh time for the cache entry */
@@ -765,6 +836,9 @@ mdcache_test_attrs_trust(mdcache_entry_t *entry, attrmask_t mask)
 	if (mask & ATTR4_FS_LOCATIONS)
 		flags |= MDCACHE_TRUST_FS_LOCATIONS;
 
+	if (mask & ATTR4_SEC_LABEL)
+		flags |= MDCACHE_TRUST_SEC_LABEL;
+
 	/* If any of the requested attributes are not valid, return. */
 	if (!test_mde_flags(entry, flags))
 		return false;
@@ -1084,4 +1158,21 @@ fsal_status_t mdcache_refresh_attrs_no_invalidate(mdcache_entry_t *entry)
 	return status;
 }
 
+static inline int avl_dmap_ck_cmpf(const struct avltree_node *lhs,
+				     const struct avltree_node *rhs)
+{
+	mdcache_dmap_entry_t *lk, *rk;
+
+	lk = avltree_container_of(lhs, mdcache_dmap_entry_t, node);
+	rk = avltree_container_of(rhs, mdcache_dmap_entry_t, node);
+
+	if (lk->ck < rk->ck)
+		return -1;
+
+	if (lk->ck == rk->ck)
+		return 0;
+
+	return 1;
+}
+
 #endif /* MDCACHE_INT_H */
