@@ -1,7 +1,7 @@
 /*
  * vim:noexpandtab:shiftwidth=8:tabstop=8:
  *
- * Copyright 2015-2017 Red Hat, Inc. and/or its affiliates.
+ * Copyright 2015-2019 Red Hat, Inc. and/or its affiliates.
  * Author: Daniel Gryniewicz <dang@redhat.com>
  *
  * This program is free software; you can redistribute it and/or
@@ -84,6 +84,7 @@ struct mdcache_fsal_module MDCACHE = {
 			.umask = 0,
 			.auth_exportpath_xdev = false,
 			.link_supports_permission_checks = true,
+			.expire_time_parent = -1,
 		}
 	}
 };
@@ -228,6 +229,13 @@ mdcache_fsal_create_export(struct fsal_module *sub_fsal, void *parse_node,
 
 	fsal_export_stack(op_ctx->fsal_export, &myself->mfe_exp);
 
+	status = dirmap_lru_init(myself);
+	if (FSAL_IS_ERROR(status)) {
+		LogMajor(COMPONENT_FSAL, "Failed to call dirmap_lru_init");
+		gsh_free(myself->name);
+		gsh_free(myself);
+		return status;
+	}
 
 	/* Set up op_ctx */
 	op_ctx->fsal_export = &myself->mfe_exp;
