@@ -1009,10 +1009,10 @@ static bool posix_get_fsid(struct fsal_filesystem *fs)
 	dev_name = blkid_devno_to_devname(mnt_stat.st_dev);
 
 	if (dev_name == NULL) {
-		LogInfo(COMPONENT_FSAL,
-			"blkid_devno_to_devname of %s failed for dev %d.%d",
-			fs->path, major(mnt_stat.st_dev),
-			minor(mnt_stat.st_dev));
+		LogDebug(COMPONENT_FSAL,
+			 "blkid_devno_to_devname of %s failed for dev %d.%d",
+			 fs->path, major(mnt_stat.st_dev),
+			 minor(mnt_stat.st_dev));
 		goto out;
 	}
 
@@ -1321,6 +1321,38 @@ int populate_posix_file_systems(bool force)
 	return retval;
 }
 
+int reload_posix_filesystems(const char *path,
+			     struct fsal_module *fsal,
+			     struct fsal_export *exp,
+			     claim_filesystem_cb claim,
+			     unclaim_filesystem_cb unclaim,
+			     struct fsal_filesystem **root_fs)
+{
+	int retval = 0;
+
+	retval = populate_posix_file_systems(true);
+
+	if (retval != 0) {
+		LogCrit(COMPONENT_FSAL,
+			"populate_posix_file_systems returned %s (%d)",
+			strerror(retval), retval);
+		return retval;
+	}
+
+	retval = claim_posix_filesystems(path, fsal, exp,
+					 claim, unclaim, root_fs);
+
+	if (retval != 0) {
+		if (retval == EAGAIN)
+			retval = ENOENT;
+		LogCrit(COMPONENT_FSAL,
+			"claim_posix_filesystems(%s) returned %s (%d)",
+			path, strerror(retval), retval);
+	}
+
+	return retval;
+}
+
 int resolve_posix_filesystem(const char *path,
 			     struct fsal_module *fsal,
 			     struct fsal_export *exp,
@@ -1344,30 +1376,22 @@ int resolve_posix_filesystem(const char *path,
 	/* second attempt to resolve file system with force option in case of
 	 * ganesha isn't during startup.
 	 */
-	if (!nfs_init.init_complete || retval != EAGAIN)
+	if (!nfs_init.init_complete || retval != EAGAIN) {
+		LogDebug(COMPONENT_FSAL,
+			 "Not trying to claim filesystems again because %s %s(%d)",
+			 nfs_init.init_complete
+				? "retval != EAGAIN"
+				: "init is not complete",
+			 strerror(retval), retval);
 		return retval;
+	}
 
 	LogDebug(COMPONENT_FSAL,
-		 "Call populate_posix_file_systems one more time");
+		 "Attempting to find a filesystem for %s, reload filesystems",
+		 path);
 
-	retval = populate_posix_file_systems(true);
-	if (retval != 0) {
-		LogCrit(COMPONENT_FSAL,
-			"populate_posix_file_systems returned %s (%d)",
-			strerror(retval), retval);
-		return retval;
-	}
-
-	retval = claim_posix_filesystems(path, fsal, exp,
-					 claim, unclaim, root_fs);
-
-	if (retval != 0) {
-		if (retval == EAGAIN)
-			retval = ENOENT;
-		LogCrit(COMPONENT_FSAL,
-			"claim_posix_filesystems(%s) returned %s (%d)",
-			path, strerror(retval), retval);
-	}
+	retval =
+	    reload_posix_filesystems(path, fsal, exp, claim, unclaim, root_fs);
 
 	return retval;
 }
@@ -1480,6 +1504,7 @@ int process_claim(const char *path,
 	struct glist_head *glist;
 	struct fsal_filesystem *fs;
 	int retval = 0;
+	bool already_claimed = this->fsal == fsal;
 
 	/* Check if the filesystem is already directly exported by some other
 	 * FSAL - note we can only get here is this is the root filesystem for
@@ -1523,9 +1548,15 @@ int process_claim(const char *path,
 		return retval;
 	}
 
-	LogDebug(COMPONENT_FSAL,
-		 "FSAL %s Claiming %s",
-		 fsal->name, this->path);
+	if (already_claimed) {
+		LogDebug(COMPONENT_FSAL,
+			 "FSAL %s Repeat Claiming %s",
+			 fsal->name, this->path);
+	} else {
+		LogInfo(COMPONENT_FSAL,
+			"FSAL %s Claiming %s",
+			fsal->name, this->path);
+	}
 
 	/* Complete the claim */
 	this->fsal = fsal;
