@@ -47,6 +47,8 @@
 #include "gsh_dbus.h"
 #include "mdcache.h"
 #endif
+#include "conf_url.h"
+#include "conf_url_rados.h"
 
 /**
  * @brief Mutex protecting shutdown flag.
@@ -431,6 +433,9 @@ static void do_shutdown(void)
 
 	LogEvent(COMPONENT_MAIN, "NFS EXIT: stopping NFS service");
 
+	rados_url_shutdown_watch();
+	config_url_shutdown();
+
 #ifdef USE_DBUS
 	/* DBUS shutdown */
 	gsh_dbus_pkgshutdown();
@@ -461,15 +466,17 @@ static void do_shutdown(void)
 
 	LogEvent(COMPONENT_MAIN, "Stopping worker threads");
 #ifdef _USE_9P
-	rc = _9p_worker_shutdown();
+	if (nfs_param.core_param.core_options & CORE_OPTION_9P) {
+		rc = _9p_worker_shutdown();
 
-	if (rc != 0) {
-		LogMajor(COMPONENT_THREAD,
-			 "Unable to shut down worker threads: %d", rc);
-		disorderly = true;
-	} else {
-		LogEvent(COMPONENT_THREAD,
-			 "Worker threads successfully shut down.");
+		if (rc != 0) {
+			LogMajor(COMPONENT_THREAD,
+				 "Unable to shut down worker threads: %d", rc);
+			disorderly = true;
+		} else {
+			LogEvent(COMPONENT_THREAD,
+				 "Worker threads successfully shut down.");
+		}
 	}
 #endif
 
