@@ -73,6 +73,7 @@
 #include "mdcache.h"
 #include "common_utils.h"
 #include "nfs_init.h"
+#include "conf_url_rados.h"
 
 /**
  * @brief init_complete used to indicate if ganesha is during
@@ -449,6 +450,9 @@ int nfs_set_param_from_conf(config_file_t parse_tree,
 		return -1;
 #endif
 
+	if (rados_url_setup_watch() != 0)
+		return -1;
+
 	LogEvent(COMPONENT_INIT, "Configuration file successfully parsed");
 
 	return 0;
