@@ -785,14 +785,7 @@ int nfs_rpc_create_chan_v41(SVCXPRT *xprt, nfs41_session_t *session,
 		goto out;
 	}
 
-	if (rpc_cb_null(chan, true) != RPC_SUCCESS)
-#ifdef EBADFD
-		code = EBADFD;
-#else				/* !EBADFD */
-		code = EBADF;
-#endif				/* !EBADFD */
-	else
-		atomic_set_uint32_t_bits(&session->flags, session_bc_up);
+	atomic_set_uint32_t_bits(&session->flags, session_bc_up);
 
  out:
 	if (code != 0) {
@@ -828,7 +821,7 @@ rpc_call_channel_t *nfs_rpc_get_chan(nfs_client_id_t *clientid, uint32_t flags)
 	if (clientid->cid_minorversion == 0) {
 		chan = &clientid->cid_cb.v40.cb_chan;
 		if (!chan->clnt) {
-			if (!nfs_rpc_create_chan_v40(clientid, flags)) {
+			if (nfs_rpc_create_chan_v40(clientid, flags)) {
 				chan = NULL;
 			}
 		}
