@@ -78,6 +78,8 @@ int nfs4_op_locku(struct nfs_argop4 *op, compound_data_t *data,
 	fsal_lock_param_t lock_desc;
 	/*  */
 	nfsstat4 nfs_status = NFS4_OK;
+	uint64_t maxfilesize =
+	    op_ctx->fsal_export->exp_ops.fs_maxfilesize(op_ctx->fsal_export);
 
 	LogDebug(COMPONENT_NFS_V4_LOCK,
 		 "Entering NFS v4 LOCKU handler ----------------------------");
@@ -166,14 +168,30 @@ int nfs4_op_locku(struct nfs_argop4 *op, compound_data_t *data,
 	}
 
 	/* Check for range overflow Remember that a length with all
-	   bits set to 1 means "lock until the end of file" (RFC3530,
-	   page 157) */
+	 * bits set to 1 means "lock until the end of file" (RFC3530,
+	 * page 157)
+	 */
 	if (lock_desc.lock_length >
 	    (STATE_LOCK_OFFSET_EOF - lock_desc.lock_start)) {
 		res_LOCKU4->status = NFS4ERR_INVAL;
 		goto out;
 	}
 
+	/* Check for range overflow past maxfilesize.  Comparing beyond 2^64 is
+	 * not possible in 64 bits precision, but off+len > maxfilesize is
+	 * equivalent to len > maxfilesize - off.  We checked 64-bit overflow
+	 * above, so treat overflowing the FS maxsize as a request to lock the
+	 * entire file.
+	 */
+	if (lock_desc.lock_length > (maxfilesize - lock_desc.lock_start)) {
+		LogDebug(COMPONENT_NFS_V4_LOCK,
+			 "LOCK past maxfilesize %"PRIx64" start %"PRIx64
+			 " length %"PRIx64,
+			 maxfilesize,
+			 lock_desc.lock_start, lock_desc.lock_length);
+		lock_desc.lock_length = 0;
+	}
+
 	LogLock(COMPONENT_NFS_V4_LOCK, NIV_FULL_DEBUG, locku_tag,
 		data->current_obj, lock_owner, &lock_desc);
 
