@@ -174,6 +174,10 @@ static nfsstat4 open4_validate_claim(compound_data_t *data,
 		break;
 
 	case CLAIM_DELEG_CUR_FH:
+		if (data->minorversion == 0)
+			status = NFS4ERR_NOTSUPP;
+		break;
+
 	case CLAIM_DELEG_PREV_FH:
 		status = NFS4ERR_NOTSUPP;
 		break;
@@ -339,7 +343,15 @@ static nfsstat4 open4_claim_deleg(OPEN4args *arg, compound_data_t *data)
 		return NFS4ERR_NOTSUPP;
 	}
 
-	assert(claim == CLAIM_DELEGATE_CUR);
+	assert((claim  == CLAIM_DELEGATE_CUR) ||
+	       (claim == CLAIM_DELEG_CUR_FH));
+
+	if (claim == CLAIM_DELEG_CUR_FH) {
+		rcurr_state =
+			&arg->claim.open_claim4_u.oc_delegate_stateid;
+		goto find_state;
+	}
+
 	utfname = &arg->claim.open_claim4_u.delegate_cur_info.file;
 	rcurr_state =
 		&arg->claim.open_claim4_u.delegate_cur_info.delegate_stateid;
@@ -371,6 +383,7 @@ static nfsstat4 open4_claim_deleg(OPEN4args *arg, compound_data_t *data)
 		return status;
 	}
 
+find_state:
 	found_state = nfs4_State_Get_Pointer(rcurr_state->other);
 
 	if (found_state == NULL) {
@@ -817,7 +830,6 @@ static void open4_ex(OPEN4args *arg,
 			res_OPEN4->status = NFS4ERR_RECLAIM_BAD;
 			goto out;
 		}
-		openflags |= FSAL_O_RECLAIM;
 		file_obj = data->current_obj;
 		break;
 
@@ -833,10 +845,10 @@ static void open4_ex(OPEN4args *arg,
 		goto out;
 
 	case CLAIM_DELEGATE_CUR:
+	case CLAIM_DELEG_CUR_FH:
 		res_OPEN4->status = open4_claim_deleg(arg, data);
 		if (res_OPEN4->status != NFS4_OK)
 			goto out;
-		openflags |= FSAL_O_RECLAIM;
 		file_obj = data->current_obj;
 		break;
 
@@ -1260,7 +1272,7 @@ int nfs4_op_open(struct nfs_argop4 *op, compound_data_t *data,
 		LogCrit(COMPONENT_NFS_V4,
 			"Impossible condition in compound data at %s:%u.",
 			__FILE__, __LINE__);
-		goto out3;
+		return res_OPEN4->status;
 	}
 
 	/* It this a known client id? */
