@@ -124,14 +124,6 @@ done:
 	server_stats_io_done(read_arg->iov[0].iov_len, read_arg->io_amount,
 			     (data->res_READ4->status == NFS4_OK) ? true :
 			     false, false);
-
-	if (data->owner != NULL) {
-		op_ctx->clientid = NULL;
-		dec_state_owner_ref(data->owner);
-	}
-
-	if (read_arg->state)
-		dec_state_t_ref(read_arg->state);
 }
 
 /**
@@ -288,6 +280,8 @@ static int nfs4_read(struct nfs_argop4 *op, compound_data_t *data,
 	READ4res * const res_READ4 = &resp->nfs_resop4_u.opread;
 	uint64_t size = 0;
 	uint64_t offset = 0;
+	uint64_t MaxRead = 0;
+	uint64_t MaxOffsetRead = 0;
 	void *bufferdata = NULL;
 	fsal_status_t fsal_status = {0, 0};
 	state_t *state_found = NULL;
@@ -296,10 +290,6 @@ static int nfs4_read(struct nfs_argop4 *op, compound_data_t *data,
 	bool anonymous_started = false;
 	state_owner_t *owner = NULL;
 	bool bypass = false;
-	uint64_t MaxRead = atomic_fetch_uint64_t(&op_ctx->ctx_export->MaxRead);
-	uint64_t MaxOffsetRead =
-			atomic_fetch_uint64_t(
-				&op_ctx->ctx_export->MaxOffsetRead);
 	struct nfs4_read_data read_data;
 	struct fsal_io_arg *read_arg = alloca(sizeof(*read_arg) +
 						sizeof(struct iovec));
@@ -480,6 +470,10 @@ static int nfs4_read(struct nfs_argop4 *op, compound_data_t *data,
 		goto out;
 	}
 
+	MaxRead = atomic_fetch_uint64_t(&op_ctx->ctx_export->MaxRead);
+	MaxOffsetRead =
+		atomic_fetch_uint64_t(&op_ctx->ctx_export->MaxOffsetRead);
+
 	/* Get the size and offset of the read operation */
 	offset = arg_READ4->offset;
 	size = arg_READ4->count;
@@ -571,6 +565,15 @@ static int nfs4_read(struct nfs_argop4 *op, compound_data_t *data,
 	if (state_open != NULL)
 		dec_state_t_ref(state_open);
 
+	if (owner != NULL) {
+		op_ctx->clientid = NULL;
+		dec_state_owner_ref(owner);
+	}
+	if (state_found != NULL) {
+		dec_state_t_ref(state_found);
+	}
+
+
 	return res_READ4->status;
 }				/* nfs4_op_read */
 
