@@ -257,6 +257,52 @@ bool nfs_grace_is_member(void)
 	return true;
 }
 
+/**
+ * @brief Return nodeid for the server
+ *
+ * If the recovery backend specifies a nodeid, return it. If it does not
+ * specify one, default to using the server's hostname.
+ *
+ * Returns 0 on success and fills out pnodeid. Caller must free the returned
+ * value with gsh_free. Returns negative POSIX error code on error.
+ */
+int nfs_recovery_get_nodeid(char **pnodeid)
+{
+	int rc;
+	long maxlen;
+	char *nodeid = NULL;
+
+	if (recovery_backend->get_nodeid) {
+		rc = recovery_backend->get_nodeid(&nodeid);
+
+		/* Return error if we got one */
+		if (rc)
+			return rc;
+
+		/* If we got a nodeid, then we're done */
+		if (nodeid) {
+			*pnodeid = nodeid;
+			return 0;
+		}
+	}
+
+	/*
+	 * Either the backend doesn't support get_nodeid or it handed back a
+	 * NULL pointer. Just use hostname.
+	 */
+	maxlen = sysconf(_SC_HOST_NAME_MAX);
+	nodeid = gsh_malloc(maxlen);
+	rc = gethostname(nodeid, maxlen);
+	if (rc != 0) {
+		LogEvent(COMPONENT_CLIENTID, "gethostname failed: %d", errno);
+		rc = -errno;
+		gsh_free(nodeid);
+	} else {
+		*pnodeid = nodeid;
+	}
+	return rc;
+}
+
 void nfs_try_lift_grace(void)
 {
 	bool in_grace = true;
@@ -360,7 +406,6 @@ static bool check_clid(nfs_client_id_t *clientid, clid_entry_t *clid_ent)
 {
 	bool ret = false;
 
-	PTHREAD_MUTEX_lock(&clientid->cid_mutex);
 
 	LogDebug(COMPONENT_CLIENTID, "compare %s to %s",
 		 clientid->cid_recov_tag, clid_ent->cl_name);
@@ -370,7 +415,6 @@ static bool check_clid(nfs_client_id_t *clientid, clid_entry_t *clid_ent)
 		     clid_ent->cl_name, PATH_MAX))
 		ret = true;
 
-	PTHREAD_MUTEX_unlock(&clientid->cid_mutex);
 	return ret;
 }
 
@@ -398,6 +442,7 @@ void  nfs4_chk_clid_impl(nfs_client_id_t *clientid, clid_entry_t **clid_ent_arg)
 	 * loop through the list and try to find this client. If we
 	 * find it, mark it to allow reclaims.
 	 */
+	PTHREAD_MUTEX_lock(&clientid->cid_mutex);
 	glist_for_each(node, &clid_list) {
 		clid_ent = glist_entry(node, clid_entry_t, cl_list);
 		if (check_clid(clientid, clid_ent)) {
@@ -414,9 +459,10 @@ void  nfs4_chk_clid_impl(nfs_client_id_t *clientid, clid_entry_t **clid_ent_arg)
 			}
 			clientid->cid_allow_reclaim = true;
 			*clid_ent_arg = clid_ent;
-			return;
+			break;
 		}
 	}
+	PTHREAD_MUTEX_unlock(&clientid->cid_mutex);
 }
 
 void  nfs4_chk_clid(nfs_client_id_t *clientid)
