@@ -377,26 +377,6 @@ void _state_del_locked(state_t *state, const char *func, int line)
 	owner = state->state_owner;
 	PTHREAD_MUTEX_unlock(&state->state_mutex);
 
-	/* Don't cleanup when ref is dropped, as this could recurse into here.
-	 * Caller must have a ref anyway.
-	 */
-	obj->state_hdl->no_cleanup = true;
-
-	/* Remove from the list of states for a particular file */
-	PTHREAD_MUTEX_lock(&state->state_mutex);
-	glist_del(&state->state_list);
-	/* Put ref for this state entry */
-	obj->obj_ops->put_ref(obj);
-	state->state_obj = NULL;
-	PTHREAD_MUTEX_unlock(&state->state_mutex);
-
-	/* We need to close the state at this point. The state will
-	 * eventually be freed and it must be closed before free. This
-	 * is the last point we have a valid reference to the object
-	 * handle.
-	 */
-	(void) obj->obj_ops->close2(obj, state);
-
 	if (owner != NULL) {
 		bool owner_retain = false;
 		struct state_nfs4_owner_t *nfs4_owner;
@@ -468,6 +448,9 @@ void _state_del_locked(state_t *state, const char *func, int line)
 	if (state->state_type == STATE_TYPE_LOCK)
 		glist_del(&state->state_data.lock.state_sharelist);
 
+	if (state->state_type == STATE_TYPE_SHARE)
+		assert(glist_empty(&state->state_data.share.share_lockstates));
+
 	/* Reset write delegated if this is a write delegation */
 	if (state->state_type == STATE_TYPE_DELEG &&
 	    state->state_data.deleg.sd_type == OPEN_DELEGATE_WRITE)
@@ -495,6 +478,26 @@ void _state_del_locked(state_t *state, const char *func, int line)
 	PTHREAD_MUTEX_unlock(&all_state_v4_mutex);
 #endif
 
+	/* Don't cleanup when ref is dropped, as this could recurse into here.
+	 * Caller must have a ref anyway.
+	 */
+	obj->state_hdl->no_cleanup = true;
+
+	/* Remove from the list of states for a particular file */
+	PTHREAD_MUTEX_lock(&state->state_mutex);
+	glist_del(&state->state_list);
+	/* Put ref for this state entry */
+	obj->obj_ops->put_ref(obj);
+	state->state_obj = NULL;
+	PTHREAD_MUTEX_unlock(&state->state_mutex);
+
+	/* We need to close the state at this point. The state will
+	 * eventually be freed and it must be closed before free. This
+	 * is the last point we have a valid reference to the object
+	 * handle.
+	 */
+	(void) obj->obj_ops->close2(obj, state);
+
 	/* Remove the sentinel reference */
 	dec_state_t_ref(state);
 
@@ -637,8 +640,22 @@ void state_nfs4_state_wipe(struct state_hdl *ostate)
 		state = glist_entry(glist, state_t, state_list);
 		if (state->state_type > STATE_TYPE_LAYOUT)
 			continue;
+		/* Skip STATE_TYPE_SHARE
+		 * It must be deleted after all the related LOCK states
+		 */
+		if (state->state_type == STATE_TYPE_SHARE)
+			continue;
 		state_del_locked(state);
 	}
+
+	/* Loop over again to delete any STATE_TYPE_SHARE */
+	glist_for_each_safe(glist, glistn, &ostate->file.list_of_states) {
+		state = glist_entry(glist, state_t, state_list);
+		if (state->state_type > STATE_TYPE_LAYOUT)
+			continue;
+		state_del_locked(state);
+	}
+
 }
 
 /**
@@ -918,16 +935,16 @@ void revoke_owner_delegs(state_owner_t *client_owner)
 
 		state_deleg_revoke(obj, state);
 
-		/* Release refs we held */
-		obj->obj_ops->put_ref(obj);
 		put_gsh_export(op_ctx->ctx_export);
 		op_ctx->ctx_export = NULL;
 		op_ctx->fsal_export = NULL;
 
-		release_root_op_context();
-
 		PTHREAD_RWLOCK_unlock(&obj->state_hdl->state_lock);
 
+		/* Release refs we held */
+		obj->obj_ops->put_ref(obj);
+
+		release_root_op_context();
 		/* Since we dropped so_mutex, we must restart the loop. */
 		goto again;
 	}
