@@ -1,7 +1,7 @@
 /*
  * vim:noexpandtab:shiftwidth=8:tabstop=8:
  *
- * Copyright 2018 Red Hat, Inc. and/or its affiliates.
+ * Copyright 2018-2019 Red Hat, Inc. and/or its affiliates.
  * Author: Jeff Layton <jlayton@redhat.com>
  *
  * This program is free software; you can redistribute it and/or
@@ -186,7 +186,7 @@ static void rados_cluster_read_clids(nfs_grace_start_t *gsp,
 
 	snprintf(rados_recov_old_oid, sizeof(rados_recov_old_oid),
 			"rec-%16.16lx:%s", rec, nodeid);
-	ret = rados_kv_traverse(rados_kv_pop_clid_entry, &args,
+	ret = rados_kv_traverse(rados_ng_pop_clid_entry, &args,
 				rados_recov_old_oid);
 	if (ret < 0)
 		LogEvent(COMPONENT_CLIENTID,
@@ -389,6 +389,12 @@ static bool rados_cluster_is_member(void)
 	return true;
 }
 
+static int rados_cluster_get_nodeid(char **pnodeid)
+{
+	*pnodeid = gsh_strdup(nodeid);
+	return 0;
+}
+
 struct nfs4_recovery_backend rados_cluster_backend = {
 	.recovery_init = rados_cluster_init,
 	.recovery_shutdown = rados_cluster_shutdown,
@@ -402,6 +408,7 @@ struct nfs4_recovery_backend rados_cluster_backend = {
 	.set_enforcing = rados_cluster_set_enforcing,
 	.grace_enforcing = rados_cluster_grace_enforcing,
 	.is_member = rados_cluster_is_member,
+	.get_nodeid = rados_cluster_get_nodeid,
 };
 
 void rados_cluster_backend_init(struct nfs4_recovery_backend **backend)
