@@ -194,8 +194,7 @@ int rados_kv_get(char *key, char *val, char *object)
 		goto out;
 	}
 
-	strncpy(val, val_out, val_len_out);
-	val[val_len_out] = '\0';
+	strlcpy(val, val_out, val_len_out+1);
 	LogDebug(COMPONENT_CLIENTID, "%s: key=%s val=%s", __func__, key, val);
 	rados_omap_get_end(iter_vals);
 out:
@@ -254,7 +253,7 @@ again:
 		if (val_len_out == 0 && key_out == NULL && val_out == NULL)
 			break;
 		start = key_out;
-		callback(key_out, val_out, args);
+		callback(key_out, val_out, val_len_out, args);
 	}
 	rados_omap_get_end(iter_vals);
 
@@ -457,7 +456,8 @@ void rados_kv_rm_clid(nfs_client_id_t *clientid)
 	clientid->cid_recov_tag = NULL;
 }
 
-void rados_kv_pop_clid_entry(char *key, char *val, struct pop_args *pop_args)
+static void rados_kv_pop_clid_entry(char *key, char *val, size_t val_len,
+				    struct pop_args *pop_args)
 {
 	int ret;
 	char *dupval;
@@ -469,7 +469,10 @@ void rados_kv_pop_clid_entry(char *key, char *val, struct pop_args *pop_args)
 	bool takeover = pop_args->takeover;
 
 	/* extract clid records */
-	dupval = gsh_strdup(val);
+	dupval = gsh_malloc(val_len +  1);
+	memcpy(dupval, val, val_len);
+	dupval[val_len] = '\0';
+
 	cl_name = strtok(dupval, "#");
 	if (!cl_name)
 		cl_name = dupval;
@@ -481,15 +484,15 @@ void rados_kv_pop_clid_entry(char *key, char *val, struct pop_args *pop_args)
 		add_rfh_entry(clid_ent, rfh_name);
 		rfh_name = strtok(NULL, "#");
 	}
-	gsh_free(dupval);
 
 	if (!old) {
-		ret = rados_kv_put(key, val, rados_recov_old_oid);
+		ret = rados_kv_put(key, dupval, rados_recov_old_oid);
 		if (ret < 0) {
 			LogEvent(COMPONENT_CLIENTID,
 				 "Failed to move %s", key);
 		}
 	}
+	gsh_free(dupval);
 
 	if (!takeover) {
 		if (old) {
