@@ -108,6 +108,13 @@ void free_fs(struct fsal_filesystem *fs);
 
 int populate_posix_file_systems(bool force);
 
+int reload_posix_filesystems(const char *path,
+			     struct fsal_module *fsal,
+			     struct fsal_export *exp,
+			     claim_filesystem_cb claim,
+			     unclaim_filesystem_cb unclaim,
+			     struct fsal_filesystem **root_fs);
+
 int resolve_posix_filesystem(const char *path,
 			     struct fsal_module *fsal,
 			     struct fsal_export *exp,
@@ -117,6 +124,8 @@ int resolve_posix_filesystem(const char *path,
 
 void release_posix_file_systems(void);
 
+void release_posix_file_system(struct fsal_filesystem *fs);
+
 int re_index_fs_fsid(struct fsal_filesystem *fs,
 		     enum fsid_type fsid_type,
 		     struct fsal_fsid__ *fsid);
