@@ -418,6 +418,8 @@ typedef uint64_t attrmask_t;
 #define ATTR4_FS_LOCATIONS  0x0000000000800000LL
 /* xattr supported */
 #define ATTR4_XATTR  0x0000000001000000LL
+/* security labels supported */
+#define ATTR4_SEC_LABEL  0x0000000002000000LL
 
 /* attributes that used for NFS v3 */
 
@@ -487,6 +489,7 @@ struct attrlist {
 					   for attributes. Settable by FSAL. */
 	fsal_fs_locations_t *fs_locations;	/*< fs locations for this
 						    object if any */
+	struct sec_label4 sec_label;
 };
 
 /******************************************************
@@ -611,6 +614,7 @@ typedef enum {
 
 typedef uint16_t fsal_openflags_t;
 
+/* ACCESS flags */
 #define FSAL_O_CLOSED     0x0000  /* Closed */
 #define FSAL_O_READ       0x0001  /* read */
 #define FSAL_O_WRITE      0x0002  /* write */
@@ -618,15 +622,22 @@ typedef uint16_t fsal_openflags_t;
 						     * explicitly or'd together
 						     * so that FSAL_O_RDWR can
 						     * be used as a mask */
-#define FSAL_O_RECLAIM         0x0008  /* open reclaim */
-#define FSAL_O_REOPEN          0x0010  /* re-open */
+
+/* Internal flags, not really NFS open flags! */
 #define FSAL_O_ANY             0x0020  /* any open file descriptor is usable */
 #define FSAL_O_TRUNC           0x0040  /* Truncate file on open */
+
+/* DENY flags */
 #define FSAL_O_DENY_READ       0x0100
 #define FSAL_O_DENY_WRITE      0x0200
 #define FSAL_O_DENY_WRITE_MAND 0x0400  /* Mandatory deny-write (i.e. NFSv4) */
 #define FSAL_O_DENY_NONE       0x0000
 
+/* Extract NFS open flags */
+#define FSAL_O_NFS_FLAGS(flags) ((flags) & \
+		(FSAL_O_RDWR | FSAL_O_DENY_READ | \
+		 FSAL_O_DENY_WRITE | FSAL_O_DENY_WRITE_MAND))
+
 enum fsal_create_mode {
 	FSAL_NO_CREATE = 0,
 	FSAL_UNCHECKED = 1,
@@ -711,6 +722,12 @@ typedef struct fsal_staticfsinfo_t {
 	bool compute_readdir_cookie;
 	bool whence_is_name;
 	bool readdir_plus;	/*< FSAL supports readdir_plus */
+	int32_t expire_time_parent; /*< Expiration time interval in
+				       seconds for parent handle.
+				       If FS gives information about parent
+				       change for a directory with an upcall,
+				       set this to -1. Else set it to some
+				       positive value. Defaults to -1. */
 } fsal_staticfsinfo_t;
 
 /**
