@@ -397,8 +397,7 @@ void SetNameHost(const char *name)
 /* Set the function name in progress. */
 void SetNameFunction(const char *nom)
 {
-	strncpy(thread_name, nom, sizeof(thread_name));
-	thread_name[sizeof(thread_name)-1] = '\0';
+	strlcpy(thread_name, nom, sizeof(thread_name));
 	if (strlen(nom) >= sizeof(thread_name))
 		LogWarn(COMPONENT_LOG,
 			"Thread name %s too long truncated to %s",
