@@ -67,6 +67,8 @@
 
 struct timespec nfs_stats_time;
 struct timespec fsal_stats_time;
+struct timespec v3_full_stats_time;
+struct timespec v4_full_stats_time;
 /**
  * @brief Exports are stored in an AVL tree with front-end cache.
  *
@@ -790,6 +792,27 @@ bool remove_one_export(struct gsh_export *export, void *state)
 	return true;
 }
 
+static void process_unexports(void)
+{
+	struct gsh_export *export;
+
+	/* Now process all the unexports */
+	while (true) {
+		export = export_take_unexport_work();
+		if (export == NULL)
+			break;
+
+		op_ctx->ctx_export = export;
+		op_ctx->fsal_export = export->fsal_export;
+
+		unexport(export);
+		put_gsh_export(export);
+
+		op_ctx->ctx_export = NULL;
+		op_ctx->fsal_export = NULL;
+	}
+}
+
 /**
  * @brief Bring down all exports in an orderly fashion.
  */
@@ -822,22 +845,34 @@ void remove_all_exports(void)
 	 */
 	(void) foreach_gsh_export(remove_one_export, true, NULL);
 
-	/* Now process all the unexports */
-	while (true) {
-		export = export_take_unexport_work();
-		if (export == NULL)
-			break;
+	process_unexports();
+	release_root_op_context();
+}
 
-		op_ctx->ctx_export = export;
-		op_ctx->fsal_export = export->fsal_export;
+static bool prune_defunct_export(struct gsh_export *exp, void *state)
+{
+	uint64_t *pgen = state;
 
-		unexport(export);
-		put_gsh_export(export);
+	if (export_is_defunct(exp, *pgen))
+		export_add_to_unexport_work_locked(exp);
+	return true;
+}
 
-		op_ctx->ctx_export = NULL;
-		op_ctx->fsal_export = NULL;
-	}
+void prune_defunct_exports(uint64_t generation)
+{
+	struct root_op_context root_op_context;
 
+	/*
+	 * Initialize req_ctx, we use NFSv4 types here to make paths show
+	 * up sanely in the logs.
+	 */
+	init_root_op_context(&root_op_context, NULL, NULL,
+				NFS_V4, 0, NFS_REQUEST);
+
+	(void)foreach_gsh_export(prune_defunct_export, true, &generation);
+
+	/* now run the work */
+	process_unexports();
 	release_root_op_context();
 }
 
@@ -1141,19 +1176,17 @@ static bool gsh_export_removeexport(DBusMessageIter *args,
 {
 	struct gsh_export *export = NULL;
 	char *errormsg;
-	bool rc = true;
+	bool rc = false;
 	bool op_ctx_set = false;
 	struct root_op_context ctx;
 
 	export = lookup_export(args, &errormsg);
-
 	if (export == NULL) {
 		LogDebug(COMPONENT_EXPORT, "lookup_export failed with %s",
 			errormsg);
 		dbus_set_error(error, DBUS_ERROR_INVALID_ARGS,
 			       "lookup_export failed with %s",
 			       errormsg);
-		rc = false;
 		goto out;
 	}
 
@@ -1161,12 +1194,23 @@ static bool gsh_export_removeexport(DBusMessageIter *args,
 		LogDebug(COMPONENT_EXPORT,
 			"Cannot remove export with id 0");
 		put_gsh_export(export);
-		rc = false;
 		dbus_set_error(error, DBUS_ERROR_INVALID_ARGS,
 			       "Cannot remove export with id 0");
 		goto out;
 	}
 
+	PTHREAD_RWLOCK_rdlock(&export->lock);
+	rc = glist_empty(&export->mounted_exports_list);
+	PTHREAD_RWLOCK_unlock(&export->lock);
+	if (!rc) {
+		LogDebug(COMPONENT_EXPORT,
+			"Cannot remove export with submounts");
+		put_gsh_export(export);
+		dbus_set_error(error, DBUS_ERROR_INVALID_ARGS,
+			       "Cannot remove export with submounts");
+		goto out;
+	}
+
 	/* Lots of obj_ops may be called during cleanup; make sure that an
 	 * op_ctx exists */
 	if (!op_ctx) {
@@ -1986,6 +2030,75 @@ static struct gsh_dbus_method reset_statistics = {
 		 END_ARG_LIST}
 };
 
+
+/**
+ * DBUS method to get NFSv3 Detailed stats
+ */
+static bool stats_v3_full(DBusMessageIter *args,
+			DBusMessage *reply,
+			DBusError *error)
+{
+	bool success = true;
+	char *errormsg = "OK";
+	DBusMessageIter iter;
+
+	dbus_message_iter_init_append(reply, &iter);
+	if (!nfs_param.core_param.enable_FULLV3STATS) {
+		success = false;
+		errormsg = "v3_full stats disabled";
+		dbus_status_reply(&iter, success, errormsg);
+		return true;
+	}
+	dbus_status_reply(&iter, success, errormsg);
+	server_dbus_v3_full_stats(&iter);
+
+	return true;
+}
+
+static struct gsh_dbus_method v3_full_statistics = {
+	.name = "GetFULLV3Stats",
+	.method = stats_v3_full,
+	.args = {STATUS_REPLY,
+		 TIMESTAMP_REPLY,
+		 V3_FULL_REPLY,
+		 MESSAGE_REPLY,
+		 END_ARG_LIST}
+};
+
+/**
+ * DBUS method to get NFSv4 Detailed stats
+ */
+static bool stats_v4_full(DBusMessageIter *args,
+			DBusMessage *reply,
+			DBusError *error)
+{
+	bool success = true;
+	char *errormsg = "OK";
+	DBusMessageIter iter;
+
+	dbus_message_iter_init_append(reply, &iter);
+	if (!nfs_param.core_param.enable_FULLV4STATS) {
+		success = false;
+		errormsg = "v4_full stats disabled";
+		dbus_status_reply(&iter, success, errormsg);
+		return true;
+	}
+	dbus_status_reply(&iter, success, errormsg);
+	server_dbus_v4_full_stats(&iter);
+
+	return true;
+}
+
+static struct gsh_dbus_method v4_full_statistics = {
+	.name = "GetFULLV4Stats",
+	.method = stats_v4_full,
+	.args = {STATUS_REPLY,
+		 TIMESTAMP_REPLY,
+		 V4_FULL_REPLY,
+		 MESSAGE_REPLY,
+		 END_ARG_LIST}
+};
+
 /**
  * DBUS method to know current status of stats counting
  */
@@ -1996,6 +2109,7 @@ static bool stats_status(DBusMessageIter *args,
 	bool success = true;
 	char *errormsg = "OK";
 	DBusMessageIter iter, nfsstatus, fsalstatus;
+	DBusMessageIter v3_full_status, v4_full_status;
 	dbus_bool_t value;
 
 	dbus_message_iter_init_append(reply, &iter);
@@ -2017,6 +2131,24 @@ static bool stats_status(DBusMessageIter *args,
 	dbus_append_timestamp(&fsalstatus, &fsal_stats_time);
 	dbus_message_iter_close_container(&iter, &fsalstatus);
 
+	/* Send info about NFSv3 Detailed stats */
+	dbus_message_iter_open_container(&iter, DBUS_TYPE_STRUCT, NULL,
+					 &v3_full_status);
+	value = nfs_param.core_param.enable_FULLV3STATS;
+	dbus_message_iter_append_basic(&v3_full_status, DBUS_TYPE_BOOLEAN,
+					&value);
+	dbus_append_timestamp(&v3_full_status, &v3_full_stats_time);
+	dbus_message_iter_close_container(&iter, &v3_full_status);
+
+	/* Send info about NFSv4 Detailed stats */
+	dbus_message_iter_open_container(&iter, DBUS_TYPE_STRUCT, NULL,
+					 &v4_full_status);
+	value = nfs_param.core_param.enable_FULLV4STATS;
+	dbus_message_iter_append_basic(&v4_full_status, DBUS_TYPE_BOOLEAN,
+					&value);
+	dbus_append_timestamp(&v4_full_status, &v4_full_stats_time);
+	dbus_message_iter_close_container(&iter, &v4_full_status);
+
 	return true;
 }
 
@@ -2028,6 +2160,7 @@ static struct gsh_dbus_method status_stats = {
 		 END_ARG_LIST}
 };
 
+
 /**
  * DBUS method to disable statistics counting
  *
@@ -2048,16 +2181,21 @@ static bool stats_disable(DBusMessageIter *args,
 	if (strcmp(stat_type, "all") == 0) {
 		nfs_param.core_param.enable_NFSSTATS = false;
 		nfs_param.core_param.enable_FSALSTATS = false;
+		nfs_param.core_param.enable_FULLV3STATS = false;
+		nfs_param.core_param.enable_FULLV4STATS = false;
 		LogEvent(COMPONENT_CONFIG,
 			 "Disabling NFS server statistics counting");
 		LogEvent(COMPONENT_CONFIG,
 			 "Disabling FSAL statistics counting");
 		/* reset all stats counters */
 		reset_fsal_stats();
+		/* resetting server stats includes v3_full & v4_full stats */
 		reset_server_stats();
 	}
 	if (strcmp(stat_type, "nfs") == 0) {
 		nfs_param.core_param.enable_NFSSTATS = false;
+		nfs_param.core_param.enable_FULLV3STATS = false;
+		nfs_param.core_param.enable_FULLV4STATS = false;
 		LogEvent(COMPONENT_CONFIG,
 			 "Disabling NFS server statistics counting");
 		/* reset server stats counters */
@@ -2070,6 +2208,20 @@ static bool stats_disable(DBusMessageIter *args,
 		/* reset fsal stats counters */
 		reset_fsal_stats();
 	}
+	if (strcmp(stat_type, "v3_full") == 0) {
+		nfs_param.core_param.enable_FULLV3STATS = false;
+		LogEvent(COMPONENT_CONFIG,
+			 "Disabling NFSv3 Detailed statistics counting");
+		/* reset v3_full stats counters */
+		reset_v3_full_stats();
+	}
+	if (strcmp(stat_type, "v4_full") == 0) {
+		nfs_param.core_param.enable_FULLV4STATS = false;
+		LogEvent(COMPONENT_CONFIG,
+			 "Disabling NFSv4 Detailed statistics counting");
+		/* reset v4_full stats counters */
+		reset_v4_full_stats();
+	}
 
 	dbus_status_reply(&iter, success, errormsg);
 	now(&timestamp);
@@ -2116,6 +2268,18 @@ static bool stats_enable(DBusMessageIter *args,
 				 "Enabling FSAL statistics counting");
 			now(&fsal_stats_time);
 		}
+		if (!nfs_param.core_param.enable_FULLV3STATS) {
+			nfs_param.core_param.enable_FULLV3STATS = true;
+			LogEvent(COMPONENT_CONFIG,
+				 "Enabling NFSv3 Detailed statistics counting");
+			now(&v3_full_stats_time);
+		}
+		if (!nfs_param.core_param.enable_FULLV4STATS) {
+			nfs_param.core_param.enable_FULLV4STATS = true;
+			LogEvent(COMPONENT_CONFIG,
+				 "Enabling NFSv4 Detailed statistics counting");
+			now(&v4_full_stats_time);
+		}
 	}
 	if (strcmp(stat_type, "nfs") == 0 &&
 			!nfs_param.core_param.enable_NFSSTATS) {
@@ -2131,6 +2295,30 @@ static bool stats_enable(DBusMessageIter *args,
 			 "Enabling FSAL statistics counting");
 		now(&fsal_stats_time);
 	}
+	if (strcmp(stat_type, "v3_full") == 0 &&
+			!nfs_param.core_param.enable_FULLV3STATS) {
+		if (!nfs_param.core_param.enable_NFSSTATS) {
+			errormsg = "First enable NFS stats counting";
+			success = false;
+		} else {
+			nfs_param.core_param.enable_FULLV3STATS = true;
+			LogEvent(COMPONENT_CONFIG,
+			 "Enabling NFSv3 Detailed statistics counting");
+			now(&v3_full_stats_time);
+		}
+	}
+	if (strcmp(stat_type, "v4_full") == 0 &&
+			!nfs_param.core_param.enable_FULLV4STATS) {
+		if (!nfs_param.core_param.enable_NFSSTATS) {
+			errormsg = "First enable NFS stats counting";
+			success = false;
+		} else {
+			nfs_param.core_param.enable_FULLV4STATS = true;
+			LogEvent(COMPONENT_CONFIG,
+			 "Enabling NFSv4 Detailed statistics counting");
+			now(&v4_full_stats_time);
+		}
+	}
 
 	dbus_status_reply(&iter, success, errormsg);
 	now(&timestamp);
@@ -2416,6 +2604,8 @@ static struct gsh_dbus_method *export_stats_methods[] = {
 	&enable_statistics,
 	&disable_statistics,
 	&status_stats,
+	&v3_full_statistics,
+	&v4_full_statistics,
 	NULL
 };
 
