@@ -1123,6 +1123,9 @@ static int export_commit_common(void *node, void *link_mem, void *self_struct,
 			return errcnt;
 		}
 
+		/* Grab config_generation for this config */
+		probe_exp->config_gen = get_parse_root_generation(node);
+
 		/* Update atomic fields */
 		update_atomic_fields(probe_exp, export);
 
@@ -1284,6 +1287,9 @@ static int export_commit_common(void *node, void *link_mem, void *self_struct,
 
 	display_clients(export);
 
+	/* Copy the generation */
+	export->config_gen = get_parse_root_generation(node);
+
 success:
 
 	(void) StrExportOptions(&dspbuf, &export->export_perms);
@@ -1700,6 +1706,9 @@ static struct config_item fsal_params[] = {
 		_struct_, options, options_set),			\
 	CONF_ITEM_BOOLBIT_SET("Disable_ACL",				\
 		false, EXPORT_OPTION_DISABLE_ACL,			\
+		_struct_, options, options_set),			\
+	CONF_ITEM_BOOLBIT_SET("Security_Label",				\
+		false, EXPORT_OPTION_SECLABEL_SET,			\
 		_struct_, options, options_set)
 
 /**
@@ -2039,6 +2048,7 @@ int reread_exports(config_file_t in_config,
 		return -1;
 	}
 
+	prune_defunct_exports(get_config_generation(in_config));
 	return num_exp;
 }
 
